
/*
References
=================================================
References point to values which are used instead. You can use references
within the structure data which is checked and also within the check conditions.
Not everything is possible, but a lot - see below.


Syntax
--------------------------------------------------
The syntax looks easy but has a lot of variations and possibilities.

    <<<source://path>>>
    <<<source://path | source://path | default>>>
    <<<source://path#{type:"integer"} | source://path | default>>>

Within the curly braces the source from which to retrieve the value is given.
The source is given in form of an URI.
Like you see in line two you may use multiple fallback URIs and also a default
value at last.
And at last in the third line you see how to add a special check condition
after an URI. If this fails the next URI is checked.

The path may also have different possibilities based on the `source` protocol
type.


Combine
--------------------------------------------------
You may also combine the resulting value(s) of the reference(s) into one
string:

    <<<host>>>:<<<port>>>

This will result in `localhost:8080` as example.
 */

(function() {
  var MAXRETRY, TIMEOUT, Worker, alternatives, arrayJoin, async, chalk, debug, exec, exists, existsObject, format, fs, fspath, handler, multiple, pathSearch, protocolMap, read, request, typePrecedence, util, vm,
    indexOf = [].indexOf || function(item) { for (var i = 0, l = this.length; i < l; i++) { if (i in this && this[i] === item) return i; } return -1; };

  debug = require('debug')('validator:reference');

  chalk = require('chalk');

  async = require('async');

  fspath = require('path');

  request = null;

  exec = null;

  vm = null;

  util = require('alinex-util');

  fs = null;

  format = null;

  Worker = null;

  TIMEOUT = 100;

  MAXRETRY = 10;

  protocolMap = {
    http: 'web',
    https: 'web'
  };

  typePrecedence = {
    env: 5,
    struct: 4,
    context: 3,
    file: 2,
    cmd: 2,
    web: 1
  };

  exists = exports.exists = function(value) {
    if (typeof value !== 'string') {
      return false;
    }
    return Boolean(value.match(/<<<[^]*>>>/));
  };

  existsObject = exports.existsObject = function(value) {
    var _, e, i, len;
    if (Array.isArray(value)) {
      for (i = 0, len = value.length; i < len; i++) {
        e = value[i];
        if (typeof e === 'string' && e.match(/<<<[^]*>>>/)) {
          return true;
        }
      }
      return false;
    }
    if (typeof value === 'object') {
      for (_ in value) {
        e = value[_];
        if (typeof e === 'string' && e.match(/<<<[^]*>>>/)) {
          return true;
        }
      }
      return false;
    }
    if (typeof value !== 'string') {
      return false;
    }
    return Boolean(value.match(/<<<[^]*>>>/));
  };

  exports.existsWait = function(worker, cb) {
    return async.retry({
      times: MAXRETRY,
      interval: TIMEOUT
    }, function(cb) {
      if (!exists(worker.value)) {
        return cb();
      }
      return cb(new Error("Reference " + worker.value + " can not be resolved"));
    }, cb);
  };

  exports.existsSchemaWait = function(worker, cb) {
    return async.retry({
      times: MAXRETRY,
      interval: TIMEOUT
    }, function(cb) {
      if (!existsObject(worker.schema)) {
        return cb();
      }
      return cb(new Error("Reference " + worker.schema + " can not be resolved"));
    }, cb);
  };

  exports.replace = function(value, worker, cb, clone) {
    var copy;
    if (clone == null) {
      clone = false;
    }
    if (!existsObject(value)) {
      return cb(null, value);
    }
    if (typeof value === 'object') {
      copy = clone ? Array.isArray(value) ? [] : {} : value;
      return async.eachOf(value, function(e, k, cb) {
        if (copy[k] == null) {
          copy[k] = e;
        }
        if (!existsObject(e)) {
          return cb();
        }
        return multiple(e, worker.path + "/" + k, worker.root, function(err, result) {
          if (err) {
            return cb(err);
          }
          copy[k] = result;
          return cb();
        });
      }, function(err) {
        if (err) {
          return cb(err);
        }
        return cb(null, copy);
      });
    } else {
      return multiple(value, worker.path, worker.root, cb);
    }
  };

  exports.replaceSchema = function(value, worker, cb, clone) {
    var copy, k, ref, schemaWorker, v;
    if (clone == null) {
      clone = false;
    }
    if (!existsObject(value)) {
      return cb(null, value);
    }
    schemaWorker = {};
    ref = worker.root;
    for (k in ref) {
      v = ref[k];
      schemaWorker[k] = v;
    }
    schemaWorker.allowData = true;
    schemaWorker.name += "#schema";
    if (typeof value === 'object') {
      copy = clone ? Array.isArray(value) ? [] : {} : value;
      return async.eachOf(value, function(e, k, cb) {
        if (copy[k] == null) {
          copy[k] = e;
        }
        if (!existsObject(e)) {
          return cb();
        }
        return multiple(e, "" + worker.path, schemaWorker, function(err, result) {
          if (err) {
            return cb(err);
          }
          copy[k] = result;
          return cb();
        });
      }, function(err) {
        if (err) {
          return cb(err);
        }
        return cb(null, copy);
      });
    } else {
      return multiple(value, worker.path, schemaWorker, cb);
    }
  };

  multiple = function(value, path, worker, cb) {
    var list;
    if (path[0] === '/') {
      path = path.slice(1);
    }
    if (debug.enabled) {
      debug(worker.name + ": /" + path + " replace " + (util.inspect(value)) + "...");
    }
    list = value.split(/(<<<[^]*?>>>)/);
    if (list.length === 3 && list[0] === '' && list[2] === '') {
      list = [list[1]];
    }
    return async.map(list, function(v, cb) {
      if (!~v.indexOf('<<<')) {
        return cb(null, v);
      }
      v = v.slice(3, -3);
      return alternatives(v, path, worker, cb);
    }, function(err, results) {
      var result;
      if (err) {
        return cb(err);
      }
      if (results.length === 1) {
        if (debug.enabled) {
          debug(worker.name + ": /" + path + " " + (util.inspect(value)) + " is replaced by " + (util.inspect(results[0])));
        }
        return cb(null, results[0]);
      }
      result = results.join('');
      if (debug.enabled) {
        debug(worker.name + ": /" + path + " " + (util.inspect(value)) + " is replaced by " + (util.inspect(result)));
      }
      return cb(null, result);
    });
  };

  alternatives = function(value, path, worker, cb) {
    var first;
    if (debug.enabled) {
      debug(chalk.grey(worker.name + ": /" + path + " resolve " + (util.inspect(value)) + "..."));
    }
    first = true;
    return async.map(value.split(/\s+\|\s+/), function(alt, cb) {
      var list;
      if (first && !~alt.indexOf('://')) {
        alt = "struct://" + alt;
      }
      first = false;
      list = util.string.rtrim(alt, '#').split(/#/);
      if (list.length === 1 && !~alt.indexOf('://')) {
        if (debug.enabled) {
          debug(chalk.grey((worker.name + ": /" + path + " " + alt + " -> use as default value").replace(/\n/, '\\n')));
        }
        return cb(null, alt);
      }
      return read(list, path, worker, function(err, result) {
        if (err) {
          return cb(err);
        }
        if (debug.enabled) {
          debug(chalk.grey((worker.name + ": /" + path + " " + alt + " -> " + (util.inspect(result))).replace(/\n/, '\\n')));
        }
        return cb(null, result);
      });
    }, function(err, results) {
      var i, len, result;
      if (err) {
        return cb(err);
      }
      for (i = 0, len = results.length; i < len; i++) {
        result = results[i];
        if (result != null) {
          return cb(null, result);
        }
      }
      return cb();
    });
  };

  read = function(list, path, worker, cb, last, data) {
    var loc, proto, ref, ref1, src, type;
    src = list.shift();
    if (!src) {
      return cb(null, worker.value);
    }
    ref = src.split(/:\/\//), proto = ref[0], loc = ref[1];
    if (loc == null) {
      loc = proto;
    }
    proto = (function() {
      switch (proto[0]) {
        case '{':
          return 'check';
        case '%':
          return 'split';
        case '/':
          return 'match';
        case '$':
          return 'parse';
        default:
          if (src.match(/^\d/)) {
            return 'range';
          } else if (!~src.indexOf('://')) {
            return 'object';
          } else {
            return proto;
          }
      }
    })();
    if (proto === 'parse' && ~loc.indexOf('$join')) {
      proto = 'join';
    }
    if (src.slice(0, +(proto.length - 1) + 1 || 9e9) !== proto && (data == null)) {
      return cb(new Error("Incorrect use of " + proto + " without data from previous element"));
    }
    switch (false) {
      case typeof data !== 'string':
        switch (proto) {
          case 'range':
            list.unshift(src);
            proto = 'split';
            loc = '%\n';
            break;
          case 'object':
            list.unshift(src);
            proto = 'parse';
            loc = '$auto';
        }
        break;
      case !Array.isArray(data):
        switch (proto) {
          case 'object':
          case 'split':
          case 'match':
            list.unshift(src);
            proto = 'join';
            loc = '$join';
        }
    }
    if ((!Array.isArray(data) && (proto === 'range' || proto === 'join')) || (typeof data !== 'string' && (proto === 'split' || proto === 'match' || proto === 'parse')) || (typeof data !== 'object' && proto === 'object')) {
      if (debug.enabled) {
        debug(chalk.magenta((worker.name + ": /" + path + " stop at part " + proto + "://" + loc + " because wrong result type").replace(/\n/, '\\n')));
      }
      return cb();
    }
    proto = proto.toLowerCase();
    type = (ref1 = protocolMap[proto]) != null ? ref1 : proto;
    if (handler[type] == null) {
      return cb(new Error("No handler for protocol " + proto + " references defined"));
    }
    if ((last != null) && typePrecedence[type] > typePrecedence[last != null]) {
      return cb(new Error(type + "-reference can not be called from " + last + "-reference for security reasons"));
    }
    if (debug.enabled) {
      debug(chalk.grey((worker.name + ": /" + path + " evaluating " + proto + "://" + loc).replace(/\n/, '\\n')));
    }
    return handler[type](proto, loc, data, path, worker, function(err, result) {
      if (err) {
        if (debug.enabled) {
          debug(chalk.magenta((worker.name + ": /" + path + " " + proto + "://" + loc + " -> failed: " + err.message).replace(/\n/, '\\n')));
        }
        return cb(err);
      }
      if (!result) {
        if (list.length && debug.enabled) {
          debug(chalk.grey((worker.name + ": /" + path + " " + proto + "://" + loc + " -> undefined").replace(/\n/, '\\n')));
        }
        return cb();
      }
      if (list.length && debug.enabled) {
        debug(chalk.grey((worker.name + ": /" + path + " " + proto + "://" + loc + " -> " + (util.inspect(result))).replace(/\n/, '\\n')));
      }
      if (!list.length) {
        return cb(null, result);
      }
      return read(list, path, worker, cb, type, result);
    });
  };

  pathSearch = function(loc, path, data, worker, cb, check) {
    var q;
    if (path == null) {
      path = '';
    }
    if (check == null) {
      check = true;
    }
    q = fspath.resolve("/" + path, loc);
    return async.retry({
      times: MAXRETRY,
      interval: TIMEOUT
    }, function(cb) {
      var ref, result;
      result = util.object.pathSearch(data, q);
      if (exists(result)) {
        return cb(new Error("Reference pointing to " + q + " can not be resolved"));
      }
      if (result && check && !(ref = q.slice(1), indexOf.call(worker.checked, ref) >= 0)) {
        return cb(new Error("Referenced value at " + q + " is not validated"));
      }
      return cb(null, result);
    }, function(err, result) {
      if (err) {
        if (debug.enabled) {
          debug(chalk.magenta(worker.name + ": /" + path + " has a circular reference at " + q + " (" + err + ")"));
        }
        return cb(err);
      }
      if (result) {
        if (debug.enabled) {
          debug(chalk.grey(worker.name + ": /" + path + " succeeded data read at " + q));
        }
        return cb(null, result);
      }
      if (debug.enabled) {
        debug(chalk.grey(worker.name + ": /" + path + " failed data read at " + q));
      }
      if (~path.indexOf('/')) {
        return pathSearch(loc, fspath.dirname(path), data, worker, cb, check);
      } else if (path) {
        return pathSearch(loc, null, data, worker, cb, check);
      }
      return cb();
    });
  };

  arrayJoin = function(data, splitter) {
    var glue, i, len, result, v;
    glue = splitter.length === 1 ? splitter[0] : splitter.shift();
    result = '';
    for (i = 0, len = data.length; i < len; i++) {
      v = data[i];
      if (Array.isArray(v)) {
        v = arrayJoin(v, splitter);
      }
      if (result) {
        result += glue;
      }
      result += v;
    }
    return result;
  };

  handler = {

    /*
    Data Sources
    -------------------------------------------------------------
    The following examples shows the different possible data sources with their URI
    syntax and usage.
    
    #3 Value Structure
    
    The `struct` protocol is used to search for the value in the current data structure.
    
    If you don't use a protocol on the first alternative it is assumed as structure
    call so you may use the shortcut:
    
        <<<name>>>            # shortcut
        <<<struct://name>>>   # same result
    
    But this is really only possible in the first alternative because in each other
    it will be assumed as default value.
    
    __Absolute path__
    
        <<<struct:///absolute/field>>>
        <<<struct:///absolute/field.0>>>
    
    Like in the first line you give the path to the value which will be used. In the
    second line `field` is an array and the first value of it will be used.
    
    __Relative path__
    
        <<<struct://relative/field>>>
        <<<struct://../relative/field>>>
    
    This will search for the `relative` node in the current path backwards and
    then for the `field` subentry  which value is used. It will look for the
    neighbor elements, the parent and it'S neighborts and so on back to root.
    
    In relative paths you can also make backreferences like in the filesystem. So
    line 2 makes no difference but line 3 of the examples goes one level up.
    
    __Matching__
    
    See below in the path locator description for the more complex search patterns.
    
    __Subchecks__
    
    Here you may also go into a file which is referenced:
    
        <<<struct://file#address.info#1>>>
    
    Searches for a field named 'file', if it is defined as type 'file' it is already
    red. Then the above call will go to the element 'address.info' and extracts the
    first line of it.
     */
    struct: function(proto, loc, data, base, worker, cb) {
      return pathSearch(loc, base, worker.value, worker, cb);
    },

    /*
    #3 Data
    
    This may only be used within the schema definuition and allows to reference data
    elements within the schema definition.
    
        <<<data://relative/path>>>
    
    The current path in the value will be kept for relative references.
    
    This allows to have names as values in your data structure which are checked to
    be defined on another level in the data structure like in the following simplified
    example.
    
    __Example:__
    
    ``` coffee
    shema =
      type: 'object'
      keys:
        list:
          type: 'array'
          entries:
            type: 'string'
        default:
          type: 'string'
          values: '<<<data://list>>>'
    
      value =
        list: ['test1', 'test2']
        default: 'test2'
    ```
     */
    data: function(proto, loc, data, base, worker, cb) {
      if (!worker.allowData) {
        return cb(null, proto + "://" + loc);
      }
      return pathSearch(loc, base, worker.value, worker, cb);
    },

    /*
    #3 Context
    
    Additional to the validating structure which have to be completely checked an
    additional context structure may be given. The values therein are not validated
    but can be used in the references.
    
    This allows you to reference to already validated or system internal information.
    
        <<<context:///absolute/* /min>>>
    
    The syntax is nearly the same as for the value structure but relative paths makes
    no sense because you don't have a base position in the structure.
    
    This is also used if you want to reference some part of the value from the schema
    definition. You can check for a valid name which is defined elsewhere in the value
    structure like follows:
    
    Schema:
    
        templates:
          type: 'object'
          entries: [
            type: 'handlebars'
          ]
        default:
          type: 'string'
          list: '<<<context:///templates>>>'
    
    context:
    
        templates:
          first: "Wellcome {{name}}"
          ongoing: "Hello {{name}}"
        default: 'ongoing'
    
    The context may also contain other information like the 'currentDir' to be used
    in the 'file' and 'command' protocols.
     */
    context: function(proto, loc, data, base, worker, cb) {
      return pathSearch(loc, null, worker.context, worker, cb, false);
    },

    /*
    #3 Environment
    
    The following syntax allows to read from an environment variable which may be
    set on start of the program or before.
    
        <<<env://MY_HOME>>>
    
    This uses the content of the `MY_HOME` environment variable.
     */
    env: function(proto, loc, data, base, worker, cb) {
      return cb(null, process.env[loc]);
    },

    /*
    #3 File Paths
    
    File paths should be given absolute because relative paths are calculated from
    the current working directory. But you can set the used base directory as context
    setting 'currentDir', too.
    
        <<<file:///etc/myvalue>>>
        <<<file:///etc/myvalue#14>>>
        <<<file:///etc/myvalue#14/5-8>>>
        <<<file:///etc/myvalue#name/min>>>
    
    This will load the content of a text file (line 1) or use only line number 14
    of the file. separated by a colon you can also specify which column (character)
    range to use.
    And in the last example line the file has to contain some type of
    structured information from which the given element path will be used.
     */
    file: function(proto, loc, data, base, worker, cb) {
      var ref;
      if (fs == null) {
        fs = require('alinex-fs');
      }
      if (((ref = worker.context) != null ? ref.currentDir : void 0) != null) {
        loc = fspath.resolve(worker.context.currentDir, loc);
      }
      return fs.realpath(loc, function(err, path) {
        if (err) {
          if (err.code === 'ENOENT') {
            return cb();
          }
          return cb(err);
        }
        return fs.readFile(path, 'utf-8', cb);
      });
    },

    /*
    #3 Web Ressources
    
    Only use a valid URL therefore:
    
        <<<http://any.server.com/service>>>
    
    It is not allowed to use a # anchor in the URL.
    But you may use the `#` anchor to access a specific line or structured element.
    
    Possible protocols are:
    
    - http like `http://domain:port/...`
    - https like `https://domain:port/...`
    
    And you may connect to UNIX Sockets like `http://unix:/absolute/unix.socket:/request/path`
    but the paths have to be absoulte.
     */
    web: function(proto, loc, data, base, worker, cb) {
      if (request == null) {
        request = require('request');
      }
      return request({
        uri: proto + "://" + loc,
        followAllRedirects: true
      }, function(err, response, body) {
        if (err) {
          if (err.message.match(/\bENOTFOUND\b/)) {
            return cb();
          }
          return cb(err);
        }
        if (response.statusCode !== 200) {
          if (response.statusCode === 404) {
            return cb();
          }
          return cb(new Error("Server send wrong return code: " + response.statusCode));
        }
        if (body == null) {
          return cb();
        }
        return cb(null, body);
      });
    },

    /*
    #3 Command
    
    The complete path will be execute as if it is typed into the command line on
    the current directory or the one given in `work.dir`.
    
        <<<cmd://date>>>
        <<<cmd:///user/local/bin/date>>>
        <<<cmd://df -h>>>
    
    It will use the value returned on STDOUT.
    
    Note: If you use pipes remove the space before or behind, because if you have
    both it is recognized as alternative reference.
    
        <<<cmd://cat test/data/poem| head -1>>>
     */
    cmd: function(proto, loc, data, base, worker, cb) {
      var opt, ref;
      if (exec == null) {
        exec = require('child_process').exec;
      }
      opt = {};
      if (((ref = worker.context) != null ? ref.currentDir : void 0) != null) {
        opt.cwd = worker.context.currentDir;
      }
      return exec(loc, opt, function(err, result) {
        if (err) {
          return cb(err);
        }
        return cb(null, result.trim());
      });
    },

    /*
    Path Locator
    -----------------------------------------------------------
    They may be used directly as the path in `struc` references or as anchor to
    get a subvalue (region).
    
    #3 Subsearch
    
    Multiple anchors are possible to specify a next subsearch like:
    
      <<<struct:///absolute.field#1>>>
      <<<file:///data/book.yml#publishing.notice#2-4>>>
    
    That means then either a # character comes up the search will use this value
    and uses the rest of the path on this.
    
    It is also possible to inject references through the referenced field like:
    
      <<<struct://file#address.info#1>>>
      file = <<<file:///myconfig.yml>>>
    
    This means that the `file` element of the structure will be used and as this
    is also a reference the value of this will first be retrieved by the reference to
    the `myconfig.yml` file. Then the result comes back the main path will be followed
    and the specific element is used.
    
    But to keep the system secure not any context can be used in another one. The
    Following list shows the precedence and can only be used top to bottom.
    
    1. environment
    2. struct
    3. context
    4. file, command
    5. web
    6. value structure or range in value are always possible
    
    Within the same level references between both are possible.
    
    This keeps the security, so that a user can not compromise the system by injecting
    references to extract internal data.
    
    To use the resulting value as datasource and only use some part of it you may
    first convert it if it is a text entry. If nothing of the following is specified
    but a range or object selection comes it will be automatically converted
    using defaults.
     */

    /*
    #3 Split
    
    A split generates a two-dimensional array (rows and columns) out of a simple text.
    It is used for range selections.
    
        %\n - split in lines and characters
        %\n//\t - split by newline and tab
        %\n//\t - split by newline and tab
        %\n//;\\s* - csv like reader with optional spaces
    
    The separator may be a simple string or an regular expression (as string).
    The resulting array has:
    
        [0] = null
        [1][0] = row 1 as text
        [1][1] = text of row 1, column 1
        ...
    
    If no other path follows better always add a `#` at the end to prevent problems
    if last separsator is a whitespace.
     */
    split: function(proto, loc, data, base, worker, cb) {
      var result, splitter;
      splitter = loc.slice(1).split('//').map(function(s) {
        return new RegExp(s);
      });
      if (splitter.length === 1) {
        splitter.push('');
      }
      result = data.split(splitter[0]).map(function(t) {
        var col;
        col = t.split(splitter[1]);
        col.unshift(t);
        return col;
      });
      result.unshift(null);
      return cb(null, result);
    },

    /*
    #3 Match
    
    Alternatively to splits you may use an regular expression on your text. You can
    use range selections also, but without column specifier.
    
        /.../ - give an regular expression `/g` is used automatically
    
    The resulting array has the found results in elements 1..n.
     */
    match: function(proto, loc, data, base, worker, cb) {
      var re;
      re = loc.match(/\/([^]*)\/(i?)/);
      re = new RegExp(re[1], re[2] + "g");
      return cb(null, data.match(re));
    },

    /*
    #3 Parse
    
    Additional to the two methods above this can do complex transformations into
    object structures to do object selections later.
    
        $js - parse javascript to object
        $json - JSON to object
        $yaml - YAML to object
        $xml - XML to object
        ...and more
        $auto - try to autodetect correct parser
    
    See [formatter](http://alinex.github.io/node-formatter) for all possible formats
    and examples of them.
     */
    parse: function(proto, loc, data, base, worker, cb) {
      var formatType;
      if (format == null) {
        format = require('alinex-format');
      }
      formatType = loc.slice(1);
      if (formatType === 'auto') {
        formatType = null;
      }
      return format.parse(data, formatType, function(err, result) {
        return cb(null, result);
      });
    },

    /*
    #3 Range Selection
    
    If the given value is a text it will be splitted into lines and characters.
    
    Within a text element you may use the following ranges:
    
        3 - specific row as string
        3-5 - specific row range as array
        3,5 - specific row and column as array
        3,5-8 - specific column range in row as array
        1-2,5-8 - specific row and column range as array
        3[2] - specific element (2nd element of third line) as string
        3[2-4] - specific elements (2nd to 4th element of third line) as array
        3[2-4,8],4[2] - and all combined ;-)
    
    The result may be a single value, an array or an array of arrays depending on the
    selected result.
     */
    range: function(proto, loc, data, base, worker, cb) {
      var col, cols, drow, i, j, l, len, len1, len2, len3, m, ref, ref1, ref2, result, row, rows, rrows;
      rows = loc.match(/\d+(?:-\d+)?(?:\[[^\]]+\])?/g);
      result = [];
      for (i = 0, len = rows.length; i < len; i++) {
        row = rows[i];
        row = row.match(/(\d+)(?:-(\d+))?(?:\[([^\]]+)\])?/);
        row.from = parseInt(row[1]);
        row.to = row[2] != null ? parseInt(row[2]) : row.from;
        cols = (ref = row[3]) != null ? ref.split(',') : void 0;
        if ((cols != null) && Array.isArray(data[1])) {
          ref1 = data.slice(row.from, +row.to + 1 || 9e9);
          for (j = 0, len1 = ref1.length; j < len1; j++) {
            drow = ref1[j];
            rrows = [];
            for (l = 0, len2 = cols.length; l < len2; l++) {
              col = cols[l];
              col = col.match(/(\d+)(?:-(\d+))?/);
              col.from = parseInt(col[1]);
              col.to = col[2] != null ? parseInt(col[2]) : col.from;
              rrows = rrows.concat(drow.slice(col.from, +col.to + 1 || 9e9));
            }
            result.push(rrows);
          }
        } else {
          ref2 = data.slice(row.from, +row.to + 1 || 9e9);
          for (m = 0, len3 = ref2.length; m < len3; m++) {
            drow = ref2[m];
            result.push(drow[0]);
          }
        }
      }
      if (!result.length) {
        return cb();
      }
      if (result.length === 1) {
        result = result[0];
      }
      if (result.length === 1) {
        result = result[0];
      }
      return cb(null, result);
    },

    /*
    #3 Object Selection
    
    If it is a structured information you may specify the path by name:
    
        name - get first element with this name
        group/sub/name - get element with path
    
    You can search by using asterisk as directory placeholder or a double asterisk to
    go multiple level depth:
    
        name/* /min - within any subelement
        name/* /* /min - within any subelement (two level depth)
        name/** /min - within any subelement in any depth
    
    You may also use regexp notation to find the correct element:
    
        name/test[AB]/min - pattern match with one missing character
        name/test\d+/min - pattern match with multiple missing characters
    
    See the [Mozilla Developer Network](https://developer.mozilla.org/de/docs/Web/
    JavaScript/Reference/Global_Objects/RegExp)
    for the possible syntax but without modifier.
     */
    object: function(proto, loc, data, base, worker, cb) {
      return cb(null, util.object.pathSearch(data, loc));
    },

    /*
    #3 Join
    
    As opposite to split and match the results maybe joined together into a single
    string using some separators.
    
        $join , - using , as separator
        $join \n//, - using newline for first level and , as separator below
        name/test\d+/min - pattern match with multiple missing characters
    
    If no other path follows better always add a `#` at the end to prevent problems
    if last separsator is a whitespace.
     */
    join: function(proto, loc, data, base, worker, cb) {
      var splitter;
      loc = loc.slice(6);
      splitter = loc ? loc.split('//') : [', '];
      return cb(null, arrayJoin(data, splitter));
    },

    /*
    #3 Checks
    
    It is possible to validate a value within the path using the validator itself.
    
        {type: 'integer'}
    
    Therefore this part of the path have to be a javascript object.
     */
    check: function(proto, loc, data, base, worker, cb) {
      var schema;
      if (vm == null) {
        vm = require('vm');
      }
      schema = vm.runInNewContext("x=" + loc);
      if (Worker == null) {
        Worker = require('./worker');
      }
      worker = new Worker("reference-" + loc, schema, null, data);
      return worker.check(function(err) {
        if (err) {
          return cb(err);
        }
        return cb(null, worker.value);
      });
    }
  };

}).call(this);

//# sourceMappingURL=reference.map