(function() {
  var Debug, Worker, async, chalk, debug, reference, util,
    bind = function(fn, me){ return function(){ return fn.apply(me, arguments); }; };

  Debug = require('debug');

  debug = Debug('validator:worker');

  chalk = require('chalk');

  async = require('async');

  util = require('alinex-util');

  reference = require('./reference');

  Worker = (function() {
    Worker.lib = {};

    Worker.inspectObject = function(obj, depth) {
      if (depth == null) {
        depth = 1;
      }
      return util.inspect(obj, {
        depth: depth,
        breakLength: 2e308
      }).replace(/\s*\n\s*/g, ' ');
    };

    Worker.load = function(type, name) {
      var error;
      if (!Worker.lib[type]) {
        try {
          Worker.lib[type] = require("../type/" + type);
          Worker.lib[type].debug = Debug("validator:" + type);
          if (debug.enabled) {
            debug("loaded " + type + " check library");
          }
        } catch (error1) {
          error = error1;
          if (debug.enabled) {
            debug(chalk.magenta(error.message));
          }
          throw new Error("Could not load library for '" + type + "' type at " + name);
        }
      }
      return Worker.lib[type];
    };

    function Worker(name1, schema1, context, value1, root) {
      var base, fn;
      this.name = name1;
      this.schema = schema1;
      this.context = context;
      this.value = value1;
      this.root = root != null ? root : this;
      this.sendSuccess = bind(this.sendSuccess, this);
      this.sendError = bind(this.sendError, this);
      this.inspectSchema = bind(this.inspectSchema, this);
      this.inspectValue = bind(this.inspectValue, this);
      this.type = this.schema.type;
      this.lib = Worker.load(this.type, this.name);
      this.debug = this.lib.debug;
      this.path = this.name.replace(/#[^.]*/g, '').split(/\./).slice(1).join('/');
      if ((base = this.root).checked == null) {
        base.checked = [];
      }
      if (fn = this.lib.init) {
        fn.call(this);
      }
    }

    Worker.prototype.describe = function(cb) {
      var sp, text;
      if (this.debug.enabled) {
        if (this.schema.title) {
          this.debug(this.name + ": " + this.schema.title);
        }
        this.debug(chalk.grey(this.name + ": describe schema " + (this.inspectSchema())));
      }
      text = '';
      if (this.schema.title) {
        text += "__" + this.schema.title + "__ ";
        sp = this.schema.title.slice(-1) === 's' && this.schema.title.slice(-2) !== 'ss' ? 'are' : 'is';
        if (this.schema.description) {
          text += sp + " " + this.schema.description + ".";
        }
        text += "\n\n";
      }
      return this.lib.describe.call(this, function(err, detail) {
        if (err) {
          return cb(err);
        }
        return cb(null, text + detail);
      });
    };

    Worker.prototype.check = function(cb) {
      if (this.debug.enabled) {
        if (this.schema.title) {
          this.debug(this.name + ": " + this.schema.title);
        }
        this.debug(chalk.grey(this.name + ": resolve possible references"));
      }
      return async.parallel([
        (function(_this) {
          return function(cb) {
            return reference.replace(_this.value, _this, function(err, value) {
              if (err) {
                return cb(err);
              }
              _this.value = value;
              return cb();
            });
          };
        })(this), (function(_this) {
          return function(cb) {
            return reference.replaceSchema(_this.schema, _this, function(err, value) {
              if (err) {
                return cb(err);
              }
              _this.schema = value;
              return cb();
            });
          };
        })(this), (function(_this) {
          return function(cb) {
            return reference.existsWait(_this, function(err) {
              if (err) {
                return cb(err);
              }
              return reference.existsSchemaWait(_this, function(err) {
                if (err) {
                  return cb(err);
                }
                _this.debug(chalk.grey(_this.name + ": check value " + (_this.inspectValue()) + " which should be " + (_this.inspectSchema())));
                return _this.lib.check.call(_this, function(err) {
                  if (err) {
                    return cb(err);
                  }
                  _this.root.checked.push(_this.path);
                  return cb();
                });
              });
            });
          };
        })(this)
      ], cb);
    };

    Worker.prototype.sub = function(name, schema, value) {
      return new Worker(name, schema, this.context, value, this.root);
    };

    Worker.prototype.inspectValue = function() {
      return Worker.inspectObject(this.value);
    };

    Worker.prototype.inspectSchema = function() {
      var i, k, keys, len, out;
      keys = Object.keys(this.schema).filter(function(e) {
        return e !== 'title' && e !== 'description';
      });
      out = {};
      for (i = 0, len = keys.length; i < len; i++) {
        k = keys[i];
        out[k] = this.schema[k];
      }
      return Worker.inspectObject(out);
    };

    Worker.prototype.sendError = function(msg, cb) {
      var err;
      err = new Error(msg + " at validation of " + this.name);
      err.worker = this;
      if (this.debug.enabled) {
        this.debug(chalk.magenta(msg));
        this.debug(chalk.grey("  validation of " + this.name));
        this.debug(chalk.grey("  value: " + (this.inspectValue.call(this))));
        this.debug(chalk.grey("  schema: " + (this.inspectSchema.call(this))));
      }
      return cb(err);
    };

    Worker.prototype.sendSuccess = function(cb) {
      if (this.debug.enabled) {
        this.debug(chalk.grey(this.name + ": succeeded with " + (this.inspectValue())));
      }
      return cb();
    };

    return Worker;

  })();

  module.exports = Worker;

}).call(this);

//# sourceMappingURL=worker.map