
/*
Array
=================================================
A list of something.

Sanitize options:
- `delimiter` - allow value text with specified list separator
 (it can also be an regular expression)

__Check options:__
- `notEmpty` - set to true if an empty array is not valid
- `minLength` - minimum number of entries
- `maxLength` - maximum number of entries
- `toArray` - convert scalar values into array

__Validating children:__
- `entries` - specification for all entries or as array for each element

__Formatting options:__
- `shuffle` - set to true to shuffle elements on validation
- `format` - one of 'simple', 'pretty', 'json'

      data = [1, 2, 3, 'a', {b: 1}, ['c', 9]]
       * simple -> "1, 2, 3, a, [object Object], c,9"
       * pretty -> "1, 2, 3, 'a', { b: 1 }, [ 'c', 9 ]"
       * json -> '[1,2,3,"a",{"b":1},["c",9]]'


Schema Specification
---------------------------------------------------
{@schema #selfcheck}
 */

(function() {
  var array, async, rules, string, util;

  async = require('async');

  util = require('alinex-util');

  string = util.string, array = util.array;

  rules = require('../helper/rules');

  exports.describe = function(cb) {
    var text;
    text = 'A list. ';
    text += rules.optional.describe.call(this);
    text = text.replace(/\. It's/, ' which is');
    if (this.schema.notEmpty) {
      text += "It's not allowed to be empty. ";
    }
    if (this.schema.delimiter != null) {
      text += "You may also give a text or RegExp using '" + this.schema.delimiter + "' as separator for the individual entries. ";
    }
    if (this.schema.unique != null) {
      text += "All values have to be unique. ";
    }
    if ((this.schema.minLength != null) && (this.schema.maxLength != null)) {
      if (this.schema.minLength === this.schema.maxLength) {
        text += "The number of entries have to be exactly " + this.schema.minLength + " elements. ";
      } else {
        text += "The number of entries have to be between " + this.schema.minLength + " and " + this.schema.maxLength + " elements. ";
      }
    } else if (this.schema.minLength != null) {
      text += "At least " + this.schema.minLength + " elements should be given. ";
    } else if (this.schema.maxLength != null) {
      text += "Not more than " + this.schema.maxLength + " elements are allowed. ";
    }
    return async.parallel([
      (function(_this) {
        return function(cb) {
          var detail, i, max, results1;
          if (_this.schema.list == null) {
            return cb();
          }
          detail = "The following entries have a specific format: ";
          max = _this.schema.list.length - 1;
          return async.map((function() {
            results1 = [];
            for (var i = 0; 0 <= max ? i <= max : i >= max; 0 <= max ? i++ : i--){ results1.push(i); }
            return results1;
          }).apply(this), function(num, cb) {
            var worker;
            worker = _this.sub(_this.name + "." + num, _this.schema.list[num]);
            return worker.describe(function(err, subtext) {
              if (err) {
                return cb(err);
              }
              return cb(null, "\n- " + num + ": " + (subtext.replace(/\n/g, '\n  ')));
            });
          }, function(err, results) {
            if (err) {
              return cb(err);
            }
            return cb(null, detail + results.join('') + '\n');
          });
        };
      })(this), (function(_this) {
        return function(cb) {
          var detail, worker;
          if (_this.schema.entries == null) {
            return cb();
          }
          if (_this.schema.list) {
            detail = "And all other entries have to be of type:\n> ";
          } else {
            detail = "Each entry has to be of type " + _this.schema.entries.type + ":\n> ";
          }
          worker = _this.sub(_this.name + "#entries", _this.schema.entries);
          return worker.describe(function(err, subtext) {
            if (err) {
              return cb(err);
            }
            return cb(null, detail + subtext.replace(/\n/g, '\n> '));
          });
        };
      })(this)
    ], (function(_this) {
      return function(err, results) {
        if (err) {
          return cb(err);
        }
        text = text.replace(/A list\. Each entry have to be/, 'A list with each entry as');
        text = (text + results.join('')).trim() + ' ';
        if (_this.schema.shuffle) {
          text += "The list will be shuffled to get a random order. ";
        }
        if (_this.schema.format != null) {
          text += "The value will be formatted as " + _this.schema.format + " list. ";
        }
        return cb(null, text);
      };
    })(this));
  };

  exports.check = function(cb) {
    var del, i, ref, results1, skip;
    skip = rules.optional.check.call(this);
    if (skip instanceof Error) {
      return cb(skip);
    }
    if (skip) {
      return cb();
    }
    if (typeof this.value === 'string' && (this.schema.delimiter != null)) {
      del = string.toRegExp(this.schema.delimiter);
      if (this.debug.enabled) {
        this.debug(this.name + ": use delimiter " + (typeof del) + " " + (util.inspect(del)));
      }
      this.value = this.value.split(del);
    }
    if (this.schema.toArray && !Array.isArray(this.value)) {
      this.value = [this.value];
    }
    if (!Array.isArray(this.value)) {
      return this.sendError("The value has to be an array", cb);
    }
    if (this.schema.unique) {
      this.value = array.unique(this.value);
    }
    if (this.schema.notEmpty && this.value.length === 0) {
      return this.sendError("An empty array/list is not allowed", cb);
    }
    if ((this.schema.minLength != null) && this.schema.minLength === this.schema.maxLength && (this.value.length !== this.schema.minLength)) {
      return this.sendError("Exactly " + this.schema.minLength + " entries are required", cb);
    } else if ((this.schema.minLength != null) && this.schema.minLength > this.value.length) {
      return this.sendError("At least " + this.schema.minLength + " entries are required as list", cb);
    } else if ((this.schema.maxLength != null) && this.schema.maxLength < this.value.length) {
      return this.sendError("Not more than " + this.schema.maxLength + " entries are allowed as list", cb);
    }
    if (!this.value.length) {
      return this.sendSuccess(cb);
    }
    return async.map((function() {
      results1 = [];
      for (var i = 0, ref = this.value.length - 1; 0 <= ref ? i <= ref : i >= ref; 0 <= ref ? i++ : i--){ results1.push(i); }
      return results1;
    }).apply(this), (function(_this) {
      return function(num, cb) {
        var ref1, worker;
        if (((ref1 = _this.schema.list) != null ? ref1[num] : void 0) != null) {
          worker = _this.sub(_this.name + "." + num, _this.schema.list[num], _this.value[num]);
        } else if (_this.schema.entries != null) {
          worker = _this.sub(_this.name + "#entries." + num, _this.schema.entries, _this.value[num]);
        } else {
          worker = _this.sub(_this.name + "#." + num, {
            type: (function() {
              switch (false) {
                case !Array.isArray(this.value[num]):
                  return 'array';
                case typeof this.value[num] !== 'object':
                  return 'object';
                default:
                  return 'any';
              }
            }).call(_this),
            optional: true
          }, _this.value[num]);
        }
        return async.setImmediate(function() {
          return worker.check(function(err) {
            return cb(err, worker.value);
          });
        });
      };
    })(this), (function(_this) {
      return function(err, result) {
        if (err) {
          return cb(err);
        }
        _this.value = result;
        if (_this.schema.shuffle) {
          util.array.shuffle(_this.value);
        }
        if (_this.schema.format) {
          switch (_this.schema.format) {
            case 'simple':
              _this.value = _this.value.join(', ');
              break;
            case 'pretty':
              _this.value = _this.value.map(function(e) {
                return util.inspect(e);
              }).join(', ');
              break;
            case 'json':
              _this.value = JSON.stringify(_this.value);
          }
        }
        return _this.sendSuccess(cb);
      };
    })(this));
  };

  exports.selfcheck = {
    title: "Array",
    description: "the array schema definitions",
    type: 'object',
    allowedKeys: true,
    keys: util.extend({
      delimiter: {
        title: "Delimiter",
        description: "the delimiter to split given string into list",
        type: 'or',
        optional: true,
        or: [
          {
            title: "Characters",
            description: "the characters to be used as explicit delimiter",
            type: 'string'
          }, {
            title: "RegExp",
            description: "the expression to split into list",
            type: 'object',
            instanceOf: RegExp
          }
        ]
      },
      toArray: {
        title: "To Array",
        description: "a flag to automatically pack other data into an array",
        type: 'boolean',
        optional: true
      },
      unique: {
        title: "Unique",
        description: "a flag to remove duplicate entries in list",
        type: 'boolean',
        optional: true
      },
      notEmpty: {
        title: "Not Empty",
        description: "a flag to not allow an empty list",
        type: 'boolean',
        optional: true
      },
      minLength: {
        title: "Minimum",
        description: "the minimum number of elements in list",
        type: 'integer',
        optional: true,
        min: 0
      },
      maxLength: {
        title: "Maximum",
        description: "the maximum number of elements in list",
        type: 'integer',
        optional: true,
        min: '<<<minLength>>>'
      },
      list: {
        title: "Specific Formats",
        description: "the schema for each element",
        type: 'array',
        optional: true,
        entries: {
          title: "Element Format",
          description: "the schema definition for a specific element",
          type: 'object',
          mandatoryKeys: ['type']
        }
      },
      entries: {
        title: "Element Format",
        description: "the general schema definition for the entries",
        type: 'object',
        mandatoryKeys: ['type'],
        optional: true
      },
      shuffle: {
        title: "Shuffle",
        description: "the list will be shuffled to get a random order",
        type: 'boolean',
        optional: true
      },
      format: {
        title: "Format",
        description: "the type of output format to use",
        type: 'string',
        values: ['simple', 'pretty', 'json'],
        optional: true
      }
    }, rules.baseSchema, {
      "default": {
        title: "Default Value",
        description: "the default value to use if nothing given",
        type: 'array',
        optional: true
      }
    })
  };

}).call(this);

//# sourceMappingURL=array.map