
/*
Byte
=================================================
To test for byte values which may contain prefixes like `18M` or `6.2 GB`.

__Sanitize options allowed:__
- `unit` - `String` unit to convert to if no number is given
- `round` - `Boolean` rounding can be set to true for arithmetic rounding
 or use `floor` or `ceil` for the corresponding methods
- `decimals` - `Integer`  number of decimal digits to round to

__Check options:__
- `min` - `Integer` the smalles allowed number
- `max` - `Integer` the biggest allowed number

This supports the units: B, Bytes, b, bps, bits


Schema Specification
---------------------------------------------------
{@schema #selfcheck}
 */

(function() {
  var Unit, math, pattern, rules, util;

  math = require('mathjs');

  util = require('alinex-util');

  rules = require('../helper/rules');

  pattern = /^[0-9]+(\.?[0-9]*) *(k|Ki|[MGTPEZY]i?)?([Bb]|bps)?$/;

  Unit = math.type.Unit;

  Unit.UNITS.bps = {
    name: 'bps',
    base: Unit.BASE_UNITS.BIT,
    dimensions: Unit.BASE_UNITS.BIT.dimensions,
    prefixes: Unit.PREFIXES.BINARY_SHORT,
    value: 1,
    offset: 0
  };

  exports.init = function() {
    var base;
    if (this.schema.decimals && (this.schema.round == null)) {
      this.schema.round = true;
    }
    if (this.schema.round && (this.schema.decimals == null)) {
      this.schema.decimals = 0;
    }
    return (base = this.schema).min != null ? base.min : base.min = 0;
  };

  exports.describe = function(cb) {
    var text, worker;
    text = 'A byte value. ';
    text += rules.optional.describe.call(this);
    text = text.replace(/\. It's/, ' which is');
    text += "If defined as a text you may use a prefix like: k, M, G, P, T, E, Z, Y also with the unit B like '12MB' or '3.7 GiB'. ";
    worker = this.sub(this.name + "#", {
      type: 'float',
      round: this.schema.round,
      decimals: this.schema.decimals,
      min: this.schema.min,
      max: this.schema.max
    });
    return worker.describe(function(err, subtext) {
      if (err) {
        return cb(err);
      }
      return cb(null, text + subtext);
    });
  };

  exports.check = function(cb) {
    var ref, ref1, skip, unit, worker;
    skip = rules.optional.check.call(this);
    if (skip instanceof Error) {
      return cb(skip);
    }
    if (skip) {
      return cb();
    }
    if (typeof this.value === 'string') {
      if (!this.value.trim().match(pattern)) {
        return this.sendError("A byte value with optional prefixes is needed", cb);
      }
      this.value = this.value.trim();
      unit = (ref = this.schema.unit) != null ? ref : this.value.match(/(b|bits|bps)$/) ? 'b' : 'B';
      if (!this.value.match(/([bB]|bps)$/)) {
        this.value += unit;
      }
      this.value = math.unit(this.value);
      this.value = this.value.toNumber(unit);
    } else if (typeof this.value !== 'number') {
      if (this.value !== (this.value | 0)) {
        return this.sendError("The given value is no byte or float number as needed", cb);
      }
    }
    worker = this.sub(this.name + "#", {
      type: ((ref1 = this.schema.unit) != null ? ref1.match(/^[kMGTPEZY]([bB]|bps)$/) : void 0) ? 'float' : 'integer',
      round: this.schema.round,
      decimals: this.schema.decimals,
      min: this.schema.min,
      max: this.schema.max
    }, this.value);
    return worker.check((function(_this) {
      return function(err) {
        if (err) {
          return cb(err);
        }
        _this.value = worker.value;
        return _this.sendSuccess(cb);
      };
    })(this));
  };

  exports.selfcheck = {
    title: "Byte",
    description: "a definition for number of bytes or bits",
    type: 'object',
    allowedKeys: true,
    keys: util.extend({
      unit: {
        title: "Source Unit",
        description: "the unit in which an only numeric value is given, will transform to base unit",
        type: 'string',
        "default": 'B',
        matches: /^[kMGTPEZY]?([bB]|bps)$/
      },
      round: {
        title: "Rounding",
        description: "the value can be rounded in different ways",
        type: 'or',
        optional: true,
        or: [
          {
            title: "Arithmetic Rounding",
            description: "a flag which allows arithmetic rounding (till 4 down, from 5 up) if set to `true`",
            type: 'boolean'
          }, {
            title: "Alternative Rounding",
            description: "the alternative rounding method to be used",
            type: 'string',
            values: ['floor', 'ceil']
          }
        ]
      },
      decimals: {
        title: "Decimals",
        description: "the number of decimal digits to round to",
        type: 'integer',
        optional: true,
        min: 0
      },
      min: {
        title: "Min Value",
        description: "the minimal value to be set",
        type: 'float',
        optional: true
      },
      max: {
        title: "Max Value",
        description: "the maximal value to be set",
        type: 'float',
        optional: true,
        min: '<<<min>>>'
      }
    }, rules.baseSchema, {
      "default": {
        title: "Default Value",
        description: "the default value to use if nothing given",
        type: 'float',
        optional: true
      }
    })
  };

}).call(this);

//# sourceMappingURL=byte.map