
/*
Datetime
=================================================
This validator will parse the given format using different technologies in nearly
all common formats:

- ISO 8601 datetimes

      '2013-02-08'
      '2013-W06-5'
      '2013-039'
      '2013-02-08 09'
      '2013-02-08T09'
      '2013-02-08 09:30'
      '2013-02-08T09:30'
      '2013-02-08 09:30:26'
      '2013-02-08T09:30:26'
      '2013-02-08 09:30:26.123'
      '2013-02-08 24:00:00.00'

- ISO 8601 time only

- ISO 8601 date only

      '2013-02-08 09'
      '2013-W06-5 09'
      '2013-039 09'

- ISO 8601 with timezone

      '2013-02-08 09+07:00'
      '2013-02-08 09-0100'
      '2013-02-08 09Z'
      '2013-02-08 09:30:26.123+07:00'

- natural language: 'today', 'tomorrow', 'yesterday', 'last friday'

- named dates

      '17 August 2013'
      '19 Aug 2013'
      '20 Aug. 2013'
      'Sat Aug 17 2013 18:40:39 GMT+0900 (JST)'

- relative dates

  - 'This Friday at 13:00'
  - '5 days ago'

- specials: 'now'

__Parse options:__
- `range` - `Boolean` the value has to be a range consisting of two dates
- `timezone` - `String` specify timezone if none given

__Check options:__
- `min` - `Integer` the date should be after
- `max` - `Integer` the date should be before

__Format options:__
- `part` - `String` with: 'date', 'time' or 'datetime'
- `format` - `String` how to format result as string
- 'locale' - `String` used for formatting
- `toTimezone` - `String` transform date/time to the given timezone

__Output formats__

If not specified it is a Date object.

If `format = 'unix'` it will be an unix timestamp (seconds since January 1, 1970).

For all other format settings a corresponding output string will be generated. Use
the aliases like ISO8601, RFC1123, RFC2822, RFC822, RFC1036 are supported and any
[moment.js](http://momentjs.com/docs/#/displaying/) format.

Also see the interval validator for time ranges without context.

The timezones may be 'America/Toronto', 'EST' or 'Eastern Standard Time' for example.


Schema Specification
---------------------------------------------------
{@schema #selfcheck}
 */

(function() {
  var alias, async, chrono, moment, rules, util, zones;

  async = require('async');

  moment = require('moment-timezone');

  chrono = require('chrono-node');

  util = require('alinex-util');

  rules = require('../helper/rules');

  moment.createFromInputFallback = function(config) {
    return config._d = (function() {
      switch (config._i.toLowerCase()) {
        case 'now':
          return new Date();
        default:
          return chrono.parseDate(config._i);
      }
    })();
  };

  zones = {
    'Eastern Standard Time': 'EST',
    'Eastern Daylight Time': 'EDT',
    'Central Standard Time': 'CST',
    'Central Daylight Time': 'CDT',
    'Mountain Standard Time': 'MST',
    'Mountain Daylight Time': 'MDT',
    'Pacific Standard Time': 'PST',
    'Pacific Daylight Time': 'PDT',
    'Central European Time': 'CET',
    'Central European Summer Time': 'CEST',
    MESZ: 'CEST',
    MEZ: 'CET'
  };

  alias = {
    datetime: {
      ISO8601: 'YYYY-MM-DDTHH:mm:ssZ',
      RFC1123: 'ddd, DD MMM YYYY HH:mm:ss z',
      RFC2822: 'ddd, DD MMM YYYY HH:mm:ss ZZ',
      RFC822: 'ddd, DD MMM YY HH:mm:ss ZZ',
      RFC1036: 'ddd, D MMM YY HH:mm:ss ZZ'
    },
    date: {
      ISO8601: 'YYYY-MM-DD'
    }
  };

  exports.init = function() {
    var base;
    return (base = this.schema).part != null ? base.part : base.part = 'datetime';
  };

  exports.describe = function(cb) {
    var add, text;
    text = 'A valid url (unified resource locator). ';
    text += rules.optional.describe.call(this);
    text = text.replace(/\. It's/, ' which is');
    text = "A " + this.schema.part + " is needed given as calendar " + this.schema.part + " or in natural language. ";
    if (this.schema.range) {
      text += "A range with start and end date is needed. ";
    }
    if (this.schema.timezone) {
      text += "If no timezone given the time is considert as " + this.schema.timezone + " time. ";
    }
    if ((this.schema.min != null) && (this.schema.max != null)) {
      text += "The " + this.schema.part + " should be between " + this.schema.min + " and " + this.schema.max + ". ";
    } else if (this.schema.min != null) {
      text += "The " + this.schema.part + " should be before " + this.schema.min + ". ";
    } else if (this.schema.max != null) {
      text += "The " + this.schema.part + " should be after " + this.schema.max + ". ";
    }
    if (this.schema.format != null) {
      add = [];
      if (this.schema.locale) {
        add.push(this.schema.locale);
      }
      if (this.schema.toTimezone) {
        add.push(this.schema.toTimezone);
      }
      text += "The " + this.schema.part + " will be converted to " + this.schema.format + " " + (add.length ? '(' + add.join(' ') + ') ' : '') + "format. ";
    }
    return cb(null, text);
  };

  exports.check = function(cb) {
    var i, len, m, p, ref, ref1, ref2, ref3, ref4, results, skip;
    skip = rules.optional.check.call(this);
    if (skip instanceof Error) {
      return cb(skip);
    }
    if (skip) {
      return cb();
    }
    if (this.schema.timezone) {
      this.schema.timezone = (ref = zones[this.schema.timezone]) != null ? ref : this.schema.timezone;
    }
    if (this.schema.range != null) {
      results = chrono.parse(this.value);
      if ((results[0].start != null) && (results[0].end != null)) {
        this.value = [results[0].start.date(), results[0].end.date()];
      } else {
        return this.sendError("The " + this.schema.part + " has to be a range", cb);
      }
    } else {
      m = this.schema.timezone ? moment.tz(this.value, this.schema.timezone) : moment(this.value);
      if (!m.isValid()) {
        return this.sendError("The given text is not parse able as date/time", cb);
      }
      this.value = m.toDate();
    }
    if (this.schema.range != null) {
      if ((this.schema.min != null) && (this.value[0] < this.schema.min) || this.value[1] < this.schema.min) {
        return this.sendError("The " + this.schema.part + " has to be at or after " + this.schema.min, cb);
      }
      if ((this.schema.max != null) && (this.value[0] > this.schema.max) || this.value[1] > this.schema.max) {
        return this.sendError("The " + this.schema.part + " has to be at or before " + this.schema.max, cb);
      }
    } else {
      if ((this.schema.min != null) && this.value < this.schema.min) {
        return this.sendError("The " + this.schema.part + " has to be at or after " + this.schema.min, cb);
      }
      if ((this.schema.max != null) && this.value > this.schema.max) {
        return this.sendError("The " + this.schema.part + " has to be at or before " + this.schema.max, cb);
      }
    }
    if (this.schema.toTimezone) {
      this.schema.toTimezone = (ref1 = zones[this.schema.toTimezone]) != null ? ref1 : this.schema.toTimezone;
    }
    if (this.schema.range != null) {
      if (this.schema.format != null) {
        if (((ref2 = alias[this.schema.part]) != null ? ref2[this.schema.format] : void 0) != null) {
          this.schema.format = alias[this.schema.part][this.schema.format];
        }
        ref3 = [0, 1];
        for (i = 0, len = ref3.length; i < len; i++) {
          p = ref3[i];
          m = moment(this.value[p]);
          if (this.schema.toTimezone) {
            m = m.tz(this.schema.timezone);
          }
          if (this.schema.locale != null) {
            m.locale(this.schema.locale);
          }
          this.value[p] = (function() {
            switch (this.schema.format) {
              case 'unix':
                return m.unix();
              default:
                return m.format(this.schema.format);
            }
          }).call(this);
        }
      }
    } else {
      if (this.schema.format != null) {
        if (((ref4 = alias[this.schema.part]) != null ? ref4[this.schema.format] : void 0) != null) {
          this.schema.format = alias[this.schema.part][this.schema.format];
        }
        m = moment(this.value);
        if (this.schema.locale != null) {
          m.locale(this.schema.locale);
        }
        if (this.schema.toTimezone) {
          m = m.tz(this.schema.toTimezone);
        }
        this.value = (function() {
          switch (this.schema.format) {
            case 'unix':
              return m.unix();
            default:
              return m.format(this.schema.format);
          }
        }).call(this);
      }
    }
    return this.sendSuccess(cb);
  };

  exports.selfcheck = {
    title: "Datetime",
    description: "a schema definition for date values",
    type: 'object',
    allowedKeys: true,
    keys: util.extend({
      range: {
        title: "Date Range",
        description: "a flag set to `true` to require a range of start and end date",
        type: 'boolean',
        optional: true
      },
      timezone: {
        title: "Timezone",
        description: "the timezone to use if nothing given",
        type: 'string',
        optional: true
      },
      part: {
        title: "Part",
        description: "the part of the full date to extract",
        type: 'string',
        optional: true,
        values: ['date', 'time', 'datetime'],
        "default": 'datetime'
      },
      min: {
        title: "Minimum Date",
        description: "the oldest allowed date",
        type: 'datetime',
        part: '<<<part>>>',
        optional: true
      },
      max: {
        title: "Maximum Date",
        description: "the newest allowed date",
        type: 'datetime',
        part: '<<<part>>>',
        optional: true,
        min: '<<<min>>>'
      },
      format: {
        title: "Format",
        description: "the moment() format string to use",
        type: 'string',
        optional: true
      },
      toTimezone: {
        title: "To Timezone",
        description: "the timezone to which to transform given dates",
        type: 'string',
        optional: true
      },
      locale: {
        title: "Locale",
        description: "the locale country code to use for formatting",
        type: 'string',
        match: /^[a-z]{2}(-[A-Z]{2})?$/,
        optional: true
      }
    }, rules.baseSchema, {
      "default": {
        title: "Default Value",
        description: "the default value to use if nothing given",
        type: 'datetime',
        optional: true
      }
    })
  };

}).call(this);

//# sourceMappingURL=datetime.map