
/*
Email
=================================================
There are a lot of crazy possibilities in the [RFC2822](https://www.ietf.org/rfc/rfc2822.txt)
which  specifies the Email format. Perhaps it came from letting different existing
email systems represented their account, to encompass anything that was valid before.

So this check will not aim to allow all emails allowed through RFC but only
those which are reasonable and commonly used.

__Sanitize options:__
- `lowerCase` domain and gmail addresses completely
- `normalize` `Boolean` remove tags, alternative domains and subdomains

__Check options:__
- `checkServer` - `Boolean` also check for working email servers
- `denyBlacklisted` - `Boolean` deny all mail servers which are currently blacklisted
- `denyGraylistes` - `Boolean` deny all mail servers which are on the untrusted lists (graylists)


Schema Specification
---------------------------------------------------
{@schema #selfcheck}
 */

(function() {
  var async, chalk, checkBlacklisted, checkGraylistes, checkMailServer, dns, dnsbl, dnsgl, format, fs, getMyIP, loadDnsbl, loadDnsgl, myIP, normalize, request, rules, util;

  async = require('async');

  chalk = require('chalk');

  fs = require('fs');

  util = require('alinex-util');

  dns = null;

  request = null;

  format = null;

  rules = require('../helper/rules');

  dnsbl = dnsgl = exports.describe = function(cb) {
    var text;
    text = 'A reasonable working email address. ';
    text += rules.optional.describe.call(this);
    text = text.replace(/\. It's/, ' which is');
    if (this.schema.lowerCase) {
      text += "It will be lowercased. ";
    }
    if (this.schema.normalize) {
      text += "Extended formats like additional domains, subdomains and tags which mostly belong to the same mailbox will be removed. ";
    }
    if (this.schema.blacklisted) {
      text += "Don't allow mail servers which are one one of the public blacklists. ";
    }
    if (this.schema.untrusted) {
      text += "Deny all mail servers which are marked as untrusted and often used for spam. ";
    }
    return cb(null, text);
  };

  exports.check = function(cb) {
    var host, local, parts, ref, ref1, skip, worker;
    skip = rules.optional.check.call(this);
    if (skip instanceof Error) {
      return cb(skip);
    }
    if (skip) {
      return cb();
    }
    if (typeof this.value !== 'string') {
      return this.sendError("A string is needed but got " + (typeof this.value) + " instead", cb);
    }
    if (this.value.length < 5) {
      return this.sendError("The given string '" + this.value + "' is too short at least 5 characters are needed", cb);
    }
    if (this.schema.lowerCase) {
      this.value = this.value.toLowerCase();
    }
    ref = parts = this.value.split(/@/), local = ref[0], host = ref[1];
    if (parts.length !== 2) {
      return this.sendError("The address is not a valid format, too few or much @-signs", cb);
    }
    if (local.length > 64) {
      return this.sendError("The local mailbox name is too long (64 chars max)", cb);
    }
    if (this.schema.normalize) {
      ref1 = normalize(host)(local, host), local = ref1[0], host = ref1[1];
    }
    worker = this.sub(this.name + "#hostname", {
      type: 'hostname'
    }, host);
    return worker.check((function(_this) {
      return function(err) {
        if (err) {
          return cb(err);
        }
        _this.value = local + "@" + host;
        if (!_this.schema.checkServer) {
          return _this.sendSuccess(cb);
        }
        if (host === 'localhost') {
          return _this.sendSuccess(cb);
        }
        return getMyIP(function(err, ip) {
          if (err) {
            if (_this.debug.enabled) {
              _this.debug(chalk.magenta(_this.name + ": could not detect own ip address"));
            }
            return _this.sendSuccess(cb);
          }
          if (dns == null) {
            dns = require('dns');
          }
          return dns.resolveMx(host, function(err, list) {
            if (list == null) {
              return _this.sendError("Could not find nameserver entry for mail server", cb);
            }
            list = list.sort(function(a, b) {
              return a.priority - b.priority;
            }).map(function(e) {
              return e.exchange;
            });
            return async.parallel([
              function(cb) {
                return checkMailServer.call(_this, list, ip, cb);
              }, function(cb) {
                return checkBlacklisted.call(_this, list, ip, cb);
              }, function(cb) {
                return checkGraylistes.call(_this, list, ip, cb);
              }
            ], function(err) {
              if (err) {
                return _this.sendError(err.message, cb);
              }
              return _this.sendSuccess(cb);
            });
          });
        });
      };
    })(this));
  };

  exports.selfcheck = {
    title: "Email",
    description: "an email schema definition",
    type: 'object',
    allowedKeys: true,
    keys: util.extend({
      lowerCase: {
        title: "Lower Case",
        description: "a flag to transform to lower case letters",
        type: 'boolean',
        optional: true
      },
      normalize: {
        title: "Normalize",
        description: "a flag to normalize some common email address variants to their base",
        type: 'boolean',
        optional: true
      },
      checkServer: {
        title: "Check Server",
        description: "a flag to also check the MX record of the destination host if possible",
        type: 'boolean',
        optional: true
      },
      denyBlacklisted: {
        title: "Deny Blacklisted Server",
        description: "a flag to deny all mail addresses of servers which are currently on any blacklist",
        type: 'boolean',
        optional: true
      },
      denyGraylistes: {
        title: "Deny Graylistes Server",
        description: "a flag to deny also mail addresses of untrusted servers which users are unverified",
        type: 'boolean',
        optional: true
      }
    }, rules.baseSchema, {
      "default": {
        title: "Default Value",
        description: "the default value to use if nothing given",
        type: 'string',
        minLength: 5,
        optional: true
      }
    })
  };

  normalize = function(host) {
    switch (false) {
      case !host.match(/^g(oogle)?mail\.com$/i):
        return function(local) {
          return [local.replace(/\.|\+.*$/g, ''), 'gmail.com'];
        };
      default:
        return function(local, host) {
          return [local.replace(/\+.*$/g, ''), host.toLowerCase()];
        };
    }
  };

  myIP = null;

  getMyIP = function(cb) {
    if (myIP) {
      return cb(null, myIP);
    }
    if (request == null) {
      request = require('request');
    }
    return request('http://ipinfo.io/ip', function(err, response, body) {
      if (err) {
        return cb(err);
      }
      if (!err && response.statusCode === 200) {
        return cb(null, body.trim());
      } else {
        return cb(new Error('could not get IP address'));
      }
    });
  };

  checkMailServer = function(list, ip, cb) {
    var net;
    if (!(list != null ? list.length : void 0)) {
      return cb(new Error("No correct responding mail server could be detected for this domain"));
    }
    net = require('net');
    return async.detect(list, (function(_this) {
      return function(domain, cb) {
        var client, res;
        if (_this.debug.enabled) {
          _this.debug(chalk.grey(_this.name + ": check mail server under " + domain));
        }
        res = '';
        client = net.connect({
          port: 25,
          host: domain
        }, function() {
          client.write("HELO " + ip + "\r\n");
          return client.end();
        });
        client.on('data', function(data) {
          return res += data.toString();
        });
        client.on('error', function(err) {
          if (_this.debug.enabled) {
            return _this.debug(chalk.magenta(err));
          }
        });
        return client.on('end', function() {
          var i, l, len, ref;
          if (_this.debug.enabled) {
            ref = res.split(/\n/);
            for (i = 0, len = ref.length; i < len; i++) {
              l = ref[i];
              _this.debug(chalk.grey(l));
            }
          }
          return cb(null, (res != null ? res.length : void 0) > 0);
        });
      };
    })(this), function(err, res) {
      if (res) {
        return cb();
      }
      return cb(new Error("No correct responding mail server could be detected for this domain"));
    });
  };

  checkBlacklisted = function(list, ip, cb) {
    if (!this.schema.denyBlacklisted) {
      return cb();
    }
    return loadDnsbl((function(_this) {
      return function(err) {
        if (err) {
          return cb(err);
        }
        return async.each(list, function(host, cb) {
          if (_this.debug.enabled) {
            _this.debug(chalk.gray(_this.name + ": check " + host + " in blacklists"));
          }
          return dns.resolve(host, function(err, addresses) {
            if (err) {
              if (_this.debug.enabled) {
                _this.debug(chalk.magenta(err.message));
              }
              return cb();
            }
            return async.each(addresses, function(address, cb) {
              var reverse;
              reverse = address.split('.').reverse().join('.');
              return async.each(dnsbl, function(bl, cb) {
                return dns.resolve(reverse + "." + bl.zone, function(err) {
                  if (err) {
                    return cb();
                  }
                  return cb(new Error("Server " + host + " (" + address + ") is found on " + bl.name + " list check at " + bl.url));
                });
              }, cb);
            }, cb);
          });
        }, cb);
      };
    })(this));
  };

  checkGraylistes = function(list, ip, cb) {
    if (!this.schema.denyGraylisted) {
      return cb();
    }
    return loadDnsgl((function(_this) {
      return function(err) {
        if (err) {
          return cb(err);
        }
        return async.each(list, function(host, cb) {
          if (_this.debug.enabled) {
            _this.debug(chalk.gray(_this.name + ": check " + host + " in graylists"));
          }
          return dns.resolve(host, function(err, addresses) {
            if (err) {
              if (_this.debug.enabled) {
                _this.debug(chalk.magenta(err.message));
              }
              return cb();
            }
            return async.each(addresses, function(address, cb) {
              var reverse;
              reverse = address.split('.').reverse().join('.');
              return async.each(dnsgl, function(bl, cb) {
                return dns.resolve(reverse + "." + bl.zone, function(err) {
                  if (err) {
                    return cb();
                  }
                  return cb(new Error("Server " + host + " (" + address + ") is found on " + bl.name + " list check at " + bl.url));
                });
              }, cb);
            }, cb);
          });
        }, cb);
      };
    })(this));
  };

  loadDnsbl = function(cb) {
    if (dnsbl) {
      return cb();
    }
    if (format == null) {
      format = require('alinex-format');
    }
    return fs.readFile(__dirname + "/data/blacklists.yaml", 'UTF8', function(err, content) {
      if (err) {
        return cb(err);
      }
      return format.parse(content, 'yaml', function(err, data) {
        var k, v;
        if (err) {
          return cb(err);
        }
        for (k in data) {
          v = data[k];
          v.name = k;
        }
        dnsbl = data;
        return cb();
      });
    });
  };

  loadDnsgl = function(cb) {
    if (dnsgl) {
      return cb();
    }
    if (format == null) {
      format = require('alinex-format');
    }
    return fs.readFile(__dirname + "/data/graylists.yaml", 'UTF8', function(err, content) {
      if (err) {
        return cb(err);
      }
      return format.parse(content, 'yaml', function(err, data) {
        var k, v;
        if (err) {
          return cb(err);
        }
        for (k in data) {
          v = data[k];
          v.name = k;
        }
        dnsgl = data;
        return cb();
      });
    });
  };

}).call(this);

//# sourceMappingURL=email.map