
/*
File
=================================================
Check the value as valid file or directory entry.

__Sanitize options:__
- `basedir` - `String` relative paths are calculated from this directory
- `resolve` - `Boolean` should the given value be resolved to a full path

__Check options:__
- `exists` - `Boolean` true to check for already existing entry
- `find` - `Array|Function` list of directories in which to search for the file
- `filetype` - `String` check against inode type: f, file, d, dir, directory, l, link


Schema Specification
---------------------------------------------------
{@schema #selfcheck}
 */

(function() {
  var async, exists, filetype, find, fs, fspath, rules, util;

  async = require('async');

  fspath = require('path');

  fs = require('alinex-fs');

  util = require('alinex-util');

  rules = require('../helper/rules');

  exports.describe = function(cb) {
    var text;
    text = 'A valid url (unified resource locator). ';
    text += rules.optional.describe.call(this);
    text = text.replace(/\. It's/, ' which is');
    if (this.schema.exists) {
      text += "The file has to exist. ";
    }
    if (this.schema.basedir) {
      text += "Relative paths are calculated from " + this.schema.basedir + ". ";
    }
    if (this.schema.resolve) {
      text += "The path will be resolved to it's absolute path. ";
    }
    if (this.schema.find) {
      text += "A search for the file will be done. ";
    }
    if (this.schema.filetype) {
      text += "The file have to be of type '" + this.schema.filetype + "'. ";
    }
    return cb(null, text);
  };

  exports.check = function(cb) {
    var basedir, ref, skip;
    skip = rules.optional.check.call(this);
    if (skip instanceof Error) {
      return cb(skip);
    }
    if (skip) {
      return cb();
    }
    if (typeof this.value !== 'string') {
      return this.sendError("A string is needed as filename", cb);
    }
    this.value = fspath.normalize(this.value);
    if (this.value.slice(-1) === '/') {
      this.value = this.value.slice(0, -1);
    }
    basedir = fspath.resolve((ref = this.schema.basedir) != null ? ref : '.');
    return find.call(this, this.value, (function(_this) {
      return function(err, found) {
        var filepath;
        if (err) {
          return cb(err);
        }
        if (!found) {
          return _this.sendError("Could not find the file " + _this.value, cb);
        }
        if (found.slice(0, 2) === './') {
          found = found.slice(2);
        }
        filepath = fspath.resolve(basedir, found);
        if (_this.schema.resolve) {
          found = filepath;
        }
        return exists.call(_this, filepath, function(err) {
          if (err) {
            return cb(err);
          }
          return filetype.call(_this, found, function(err) {
            if (err) {
              return cb(err);
            }
            _this.value = found;
            return _this.sendSuccess(cb);
          });
        });
      };
    })(this));
  };

  exports.selfcheck = {
    title: "File",
    description: "a file schema definition",
    type: 'object',
    allowedKeys: true,
    keys: util.extend({
      basedir: {
        title: "Base Directory",
        description: "the directory to use for relative link resolving",
        type: 'string',
        "default": '.'
      },
      resolve: {
        title: "Resolve",
        description: "a flag to resolve relative links to absolute ones",
        type: 'boolean',
        "default": false
      },
      exists: {
        title: "Exists",
        description: "a flag if the file have to exist",
        type: 'boolean',
        "default": false
      },
      find: {
        title: "Find",
        description: "the directories in which to search for the file",
        type: 'array',
        toArray: true,
        optional: true,
        entries: {
          title: "Find Directories",
          description: "the directories in which to search for the file",
          type: 'string'
        }
      },
      filetype: {
        title: "File Type",
        description: "the type, the file should have",
        type: 'string',
        lowerCase: true,
        values: ['f', 'file', 'd', 'dir', 'directory', 'l', 'link', 'fifo', 'pipe', 'p', 'socket', 's'],
        optional: true
      }
    }, rules.baseSchema, {
      "default": {
        title: "Default Value",
        description: "the default value to use if nothing given",
        type: 'string',
        optional: true
      }
    })
  };

  find = function(value, cb) {
    var search;
    if (!this.schema.find) {
      return cb(null, value);
    }
    search = typeof this.schema.find === 'function' ? this.schema.find() : this.schema.find;
    if (!(search != null ? search.length : void 0)) {
      return this.sendError("Wrong find option, array is needed for file validation", cb);
    }
    return async.map(search, (function(_this) {
      return function(dir, cb) {
        if (_this.debug.enabled) {
          _this.debug(_this.name + ": search in " + dir);
        }
        return fs.find(dir, {
          filter: {
            include: _this.value
          }
        }, function(err, list) {
          return cb(null, list);
        });
      };
    })(this), function(err, lists) {
      var i, len, list;
      for (i = 0, len = lists.length; i < len; i++) {
        list = lists[i];
        if (list != null ? list.length : void 0) {
          return cb(null, list[0]);
        }
      }
      return cb();
    });
  };

  exists = function(value, cb) {
    if (!(this.schema.exists || this.schema.filetype)) {
      return cb();
    }
    return fs.exists(value, (function(_this) {
      return function(exists) {
        if (exists) {
          return cb();
        }
        return _this.sendError("The given file has to exist", cb);
      };
    })(this));
  };

  filetype = function(value, cb) {
    if (!this.schema.filetype) {
      return cb();
    }
    return fs.lstat(value, (function(_this) {
      return function(err, stats) {
        if (err) {
          return cb(err);
        }
        switch (_this.schema.filetype) {
          case 'file':
          case 'f':
            if (stats.isFile()) {
              return cb();
            }
            if (_this.debug.enabled) {
              _this.debug(_this.name + ": skip " + value + " because not a file entry");
            }
            break;
          case 'directory':
          case 'dir':
          case 'd':
            if (stats.isDirectory()) {
              return cb();
            }
            if (_this.debug.enabled) {
              _this.debug(_this.name + ": skip " + value + " because not a directory entry");
            }
            break;
          case 'link':
          case 'l':
            if (stats.isSymbolicLink()) {
              return cb();
            }
            if (_this.debug.enabled) {
              _this.debug(_this.name + ": skip " + value + " because not a link entry");
            }
            break;
          case 'fifo':
          case 'pipe':
          case 'p':
            if (stats.isFIFO()) {
              return cb();
            }
            if (_this.debug.enabled) {
              _this.debug(_this.name + ": skip " + value + " because not a FIFO entry");
            }
            break;
          case 'socket':
          case 's':
            if (stats.isSocket()) {
              return cb();
            }
            if (_this.debug.enabled) {
              _this.debug(_this.name + ": skip " + value + " because not a socket entry");
            }
        }
        return _this.sendError("The given file is not a " + _this.schema.filetype + " entry", cb);
      };
    })(this));
  };

}).call(this);

//# sourceMappingURL=file.map