
/*
Float
=================================================
This will check a floating point number against a precise definition in which
range it may be.

Sanitize options allowed:
- `sanitize` - `Boolean` remove invalid characters
- `unit` - `String` unit to convert to if no number is given
- `round` - `Boolean|String` rounding of float can be set to true for arithmetic rounding
 or use `floor` or `ceil` for the corresponding methods
- `decimals` - `Integer` number of decimal digits to round to

Check options:
- `min` - `Numeric` the smallest allowed number
- `max` - `Numeric` the biggest allowed number

Format options:
- `toUnit` - `String` unit to convert value to
- `format` - `String` format number as string
- 'locale' - `String` locale format to use for string output

The result will be a `Number` or `String` representation of a number depending on
the above parameters.

#3 Additional Possibilities

Use the {@link or.coffee} type to allow multiple ranges.

#3 Alternative Number Checks

You may also use:
- {@link integer.coffee}
- {@link percent.coffee}
- {@link byte.coffee}


Schema Specification
---------------------------------------------------
{@schema #selfcheck}
 */

(function() {
  var initMath, math, rules, util;

  math = null;

  util = require('alinex-util');

  rules = require('../helper/rules');

  initMath = function() {
    if (math) {
      return;
    }
    math = require('mathjs');
    math.type.Unit.BASE_UNITS.FREQUENCY = {};
    return math.type.Unit.UNITS.hz = {
      name: 'Hz',
      base: math.type.Unit.BASE_UNITS.FREQUENCY,
      prefixes: math.type.Unit.PREFIXES.SHORT,
      value: 1,
      offset: 0
    };
  };

  exports.init = function() {
    if (this.schema.decimals && (this.schema.round == null)) {
      this.schema.round = true;
    }
    if (this.schema.round && (this.schema.decimals == null)) {
      return this.schema.decimals = 0;
    }
  };

  exports.describe = function(cb) {
    var text, type;
    text = "A numeric floating point number. ";
    text += rules.optional.describe.call(this);
    text = text.replace(/\. It's/, ' which is');
    if (this.schema.sanitize) {
      text += "Invalid characters will be removed from text. ";
    }
    if (this.schema.unit) {
      text += "If no other unit given the number will be interpreted as " + this.schema.unit + ". ";
    }
    if (this.schema.round) {
      type = (function() {
        switch (this.schema.round) {
          case 'ceil':
            return 'to ceil';
          case 'floor':
            return 'to floor';
          default:
            return 'arithḿeticaly';
        }
      }).call(this);
      text += "The value will be rounded " + type + " to " + this.schema.decimals + " decimals. ";
    }
    if ((this.schema.min != null) && (this.schema.max != null)) {
      text += "The value should be between " + this.schema.min + " and " + this.schema.max + ". ";
    } else if (this.schema.min != null) {
      text += "The value should be greater than " + this.schema.min + ". ";
    } else if (this.schema.max != null) {
      text += "The value should be lower than " + this.schema.max + ". ";
    }
    if (this.schema.toUnit) {
      text += "The number will be formatted in unit '" + this.schema.toUnit + "'. ";
    }
    if (this.schema.format) {
      text = text.replace(/\. $/, " and it will be written like '" + this.schema.format + "'. ");
    }
    return cb(null, text);
  };

  exports.check = function(cb) {
    var exp, numeral, p, ref, ref1, skip, v;
    skip = rules.optional.check.call(this);
    if (skip instanceof Error) {
      return cb(skip);
    }
    if (skip) {
      return cb();
    }
    if (this.schema.unit != null) {
      initMath();
      if (typeof this.value === 'number' || (typeof this.value === 'string' && this.value.match(/\d$/))) {
        this.value = "" + this.value + this.schema.unit;
      }
      this.value = math.unit(this.value);
      this.value = this.value.toNumber(this.schema.unit);
    }
    if (this.schema.sanitize && typeof this.value === 'string') {
      this.value = this.value.replace(/^.*?([-+]?\d+\.?\d*).*?$/, '$1');
    }
    if (typeof this.value === 'string' && this.value.length) {
      if (this.value.length) {
        this.value = Number(this.value);
      }
    }
    if (this.schema.round) {
      exp = Math.pow(10, this.schema.decimals);
      this.value = this.value * exp;
      this.value = (function() {
        switch (this.schema.round) {
          case 'ceil':
            return Math.ceil(this.value);
          case 'floor':
            return Math.floor(this.value);
          default:
            return Math.round(this.value);
        }
      }).call(this);
      this.value = this.value / exp;
    }
    if (!(!isNaN(parseFloat(this.value)) && isFinite(this.value))) {
      return this.sendError("The given value is no number as needed", cb);
    }
    if ((this.schema.min != null) && this.value < this.schema.min) {
      return this.sendError("The value is to low, it has to be at least " + this.schema.min, cb);
    }
    if ((this.schema.max != null) && this.value > this.schema.max) {
      return this.sendError("The value is to high, it has to be at " + this.schema.max + " or lower", cb);
    }
    if (this.schema.toUnit) {
      initMath();
      this.value = math.unit(this.value, (ref = this.schema.unit) != null ? ref : this.schema.toUnit);
      if (this.value.units[0].unit.name === 's' && this.value.toNumber('s') > 120) {
        this.value = this.value.to('min');
      }
      if (this.value.units[0].unit.name === 'min' && this.value.toNumber('min') > 120) {
        this.value = this.value.to('h');
      }
      if (this.value.units[0].unit.name === 'h' && this.value.toNumber('h') > 48) {
        this.value = this.value.to('day');
      }
      if (this.schema.format) {
        numeral = require('numeral');
        if (this.schema.locale) {
          try {
            numeral.language(this.schema.locale, require("numeral/languages/" + this.schema.locale));
            numeral.language(this.schema.locale);
          } catch (error) {}
        }
        ref1 = this.value.format().split(/[ ]/), v = ref1[0], p = ref1[1];
        this.value = numeral(v).format(this.schema.format) + ' ' + p;
        if (this.schema.locale) {
          numeral.language('en');
        }
      } else {
        this.value = this.value.format();
      }
    } else if (this.schema.format) {
      numeral = require('numeral');
      if (this.schema.locale) {
        try {
          numeral.language(this.schema.locale, require("numeral/languages/" + this.schema.locale));
          numeral.language(this.schema.locale);
        } catch (error) {}
      }
      this.value = numeral(this.value).format(this.schema.format);
      if (this.schema.locale) {
        numeral.language('en');
      }
    }
    return this.sendSuccess(cb);
  };

  exports.selfcheck = {
    title: "Float",
    description: "a float schema definition",
    type: 'object',
    allowedKeys: true,
    keys: util.extend({
      sanitize: {
        title: "Sanitize",
        description: "a flag which allows removing of non numeric characters before evaluating",
        type: 'boolean',
        optional: true
      },
      unit: {
        title: "Source Unit",
        description: "the unit in which an only numeric value is given, will transform to base unit",
        type: 'string',
        optional: true,
        minLength: 1
      },
      round: {
        title: "Rounding",
        description: "the value can be rounded in different ways",
        type: 'or',
        optional: true,
        or: [
          {
            title: "Arithmetic Rounding",
            description: "a flag which allows arithmetic rounding (till 4 down, from 5 up) if set to `true`",
            type: 'boolean'
          }, {
            title: "Alternative Rounding",
            description: "the alternative rounding method to be used",
            type: 'string',
            values: ['floor', 'ceil']
          }
        ]
      },
      decimals: {
        title: "Decimals",
        description: "the number of decimal digits to round to",
        type: 'integer',
        optional: true,
        min: 0
      },
      min: {
        title: "Min Value",
        description: "the minimal value to be set",
        type: 'float',
        optional: true
      },
      max: {
        title: "Max Value",
        description: "the maximal value to be set",
        type: 'float',
        optional: true,
        min: '<<<min>>>'
      },
      toUnit: {
        title: "Result Unit",
        description: "the unit to which to transform the value",
        type: 'string',
        optional: true
      },
      format: {
        title: "Format",
        description: "the numerical output format to use",
        type: 'string',
        optional: true
      },
      locale: {
        title: "Locale",
        description: "the language to be used for locale specific number format",
        type: 'string',
        match: /^[a-z]{2}(?:-[A-Z]{2})?$/,
        optional: true
      }
    }, rules.baseSchema, {
      "default": {
        title: "Default Value",
        description: "the default value to use if nothing given",
        type: 'float',
        optional: true
      }
    })
  };

}).call(this);

//# sourceMappingURL=float.map