
/*
Handlebars
=================================================
Validate a possible handlebar template and return the function to directly use it.

A template function or template source or normal text are all valid and result in
returning a function which may be called with a context will return the template's
resulting text.

A text which may contain [handlebars syntax](http://alinex.github.io/develop/lang/handlebars.html)
will be compiled into a function which if called with the context
object will return the resulting text.

``` coffee
validator.check
  name: 'test'        # name to be displayed in errors (optional)
  value: 'hello {{name}}' # value to check
  schema:             # definition of checks
    type: 'handlebars'
, (err, result) ->
   * then use it
  console.log result
    name: 'alex'
   * this will output 'hello alex'
```

Within the handlebars templates you may use:

- [builtin helpers](http://alinex.github.io/develop/lang/handlebars.html#built-in-helpers)
- additional [handlebars](http://alinex.github.io/node-handlebars) helpers


Schema Specification
---------------------------------------------------
{@schema #selfcheck}
 */

(function() {
  var handlebars, rules, util;

  handlebars = require('handlebars');

  util = require('alinex-util');

  rules = require('../helper/rules');

  require('alinex-handlebars').register(handlebars);

  exports.describe = function(cb) {
    var text;
    text = 'A valid text which may contain handlebar syntax and variables. ';
    text += rules.optional.describe.call(this);
    text = text.replace(/\. It's/, ' which is');
    return cb(null, text);
  };

  exports.check = function(cb) {
    var fn, skip, template, v;
    skip = rules.optional.check.call(this);
    if (skip instanceof Error) {
      return cb(skip);
    }
    if (skip) {
      return cb();
    }
    if (typeof this.value === 'function') {
      return this.sendSuccess(cb);
    }
    if (typeof this.value !== 'string') {
      return this.sendError("The given value is no integer as needed", cb);
    }
    if (this.value.match(/\{\{.*?\}\}/)) {
      if (this.debug.enabled) {
        this.debug(this.name + ": compile handlebars");
      }
      template = handlebars.compile(this.value);
      fn = (function(_this) {
        return function(context) {
          if (_this.debug.enabled) {
            _this.debug(_this.name + ": execute template with " + (util.inspect(context)));
          }
          return template(context);
        };
      })(this);
    } else {
      v = this.value;
      fn = function() {
        return v;
      };
    }
    this.value = fn;
    return this.sendSuccess(cb);
  };

  exports.selfcheck = {
    title: "Handlebars",
    description: "a handlebars schema definition",
    type: 'object',
    allowedKeys: true,
    keys: util.extend({}, rules.baseSchema, {
      "default": {
        title: "Default Value",
        description: "the default value to use if nothing given",
        type: 'or',
        optional: true,
        or: [
          {
            title: "Template",
            description: "the default template text to use",
            type: 'string'
          }, {
            title: "Function",
            description: "the default function to use",
            type: 'function'
          }
        ]
      }
    })
  };

}).call(this);

//# sourceMappingURL=handlebars.map