
/*
Interval
=================================================
A time interval may be given:

- directly as number
- in a string with days, minutes and seconds: `1d 3h 12m 10s 400ms`
- in a time format: `03:20`, `02:18:10.5`

Sanitize options allowed:
- `unit` - `String` type of unit to convert if not integer given
- `round` - `Boolean` rounding can be set to true for arithmetic rounding
 or use `floor` or `ceil` for the corresponding methods
- `decimals` - `Integer` number of decimal digits to round to (defaults to 2)

Check options:
- `min` - `Integer` the smallest allowed number
- `max` - `Integer` the biggest allowed number


Schema Specification
---------------------------------------------------
{@schema #selfcheck}
 */

(function() {
  var rules, util;

  util = require('alinex-util');

  rules = require('../helper/rules');

  exports.init = function() {
    if (this.schema.decimals && (this.schema.round == null)) {
      this.schema.round = true;
    }
    if (this.schema.round && (this.schema.decimals == null)) {
      return this.schema.decimals = 0;
    }
  };

  exports.describe = function(cb) {
    var text, worker;
    text = "A time interval as float, in time format or as text which may use a combination of values with the units: ms, s, m, h, d. ";
    text += rules.optional.describe.call(this);
    text = text.replace(/\. It's/, ' which is');
    if (this.schema.unit) {
      text += "The result will be given as the number of " + this.schema.unit + ". ";
    }
    worker = this.sub(this.name + "#", {
      type: 'float',
      round: this.schema.round,
      decimals: this.schema.decimals,
      min: this.schema.min,
      max: this.schema.max
    });
    return worker.describe(function(err, subtext) {
      if (err) {
        return cb(err);
      }
      return cb(null, text + subtext);
    });
  };

  exports.check = function(cb) {
    var skip, worker;
    skip = rules.optional.check.call(this);
    if (skip instanceof Error) {
      return cb(skip);
    }
    if (skip) {
      return cb();
    }
    worker = this.sub(this.name + "#", {
      type: 'or',
      or: [
        {
          type: 'float'
        }, {
          type: 'string',
          match: /^\d\d?:\d\d?(:\d\d?)?(\.\d+)?$/
        }, {
          type: 'string',
          match: /^(([+-]?\d+(?:\.\d+)?)\s*([smhd]|ms)\s*)*$/
        }
      ]
    }, this.value);
    return worker.check((function(_this) {
      return function(err) {
        var parsed, parts, ref, unit;
        if (err) {
          return cb(err);
        }
        _this.value = worker.value;
        if (typeof _this.value === 'string') {
          if (_this.value.trim().match(/^(\d\d?)(:\d\d?)(:\d\d?)?(\.\d+)?$/)) {
            parts = _this.value.split(':');
            _this.value = parts[0] + "h " + parts[1] + "m";
            if (parts.length === 3) {
              _this.value += " " + parts[2] + "s";
            }
          }
          parsed = util.number.parseMSeconds(_this.value);
          unit = (ref = _this.schema.unit) != null ? ref : 'ms';
          if (unit !== 'ms') {
            parsed /= (function() {
              switch (unit) {
                case 's':
                  return 1000;
                case 'm':
                  return 1000 * 60;
                case 'h':
                  return 1000 * 60 * 60;
                case 'd':
                  return 1000 * 60 * 60 * 24;
              }
            })();
          }
          _this.value = parsed;
        }
        worker = _this.sub(_this.name + "#", {
          type: 'float',
          round: _this.schema.round,
          decimals: _this.schema.decimals,
          min: _this.schema.min,
          max: _this.schema.max
        }, _this.value);
        return worker.check(function(err) {
          if (err) {
            return cb(err);
          }
          _this.value = worker.value;
          return _this.sendSuccess(cb);
        });
      };
    })(this));
  };

  exports.selfcheck = {
    title: "Interval",
    description: "an interval schema definition",
    type: 'object',
    allowedKeys: true,
    keys: util.extend({
      unit: {
        title: "Source Unit",
        description: "the unit in which an only numeric value is given, will transform to base value",
        type: 'string',
        optional: true,
        values: ['d', 'h', 'm', 's', 'ms']
      },
      round: {
        title: "Rounding",
        description: "the value can be rounded in different ways",
        type: 'or',
        optional: true,
        or: [
          {
            title: "Arithmetic Rounding",
            description: "a flag which allows arithmetic rounding (till 4 down, from 5 up) if set to `true`",
            type: 'boolean'
          }, {
            title: "Alternative Rounding",
            description: "the alternative rounding method to be used",
            type: 'string',
            values: ['floor', 'ceil']
          }
        ]
      },
      decimals: {
        title: "Decimals",
        description: "the number of decimal digits to round to",
        type: 'integer',
        optional: true,
        min: 0
      },
      min: {
        title: "Min Value",
        description: "the minimal value to be set",
        type: 'float',
        optional: true
      },
      max: {
        title: "Max Value",
        description: "the maximal value to be set",
        type: 'float',
        optional: true,
        min: '<<<min>>>'
      }
    }, rules.baseSchema, {
      "default": {
        title: "Default Value",
        description: "the default value to use if nothing given",
        type: 'float',
        optional: true
      }
    })
  };

}).call(this);

//# sourceMappingURL=interval.map