
/*
IP Address
=================================================
The value has to be an IP address.

Check options:
- `optional` - `Boolean` the value must not be present (will return null)
- `default` - `String` the value to use if none given
- `version` - `String` one of 'ipv4' or 'ipv6' and the value will be converted, if possible
- `allow` - `Array` the allowed ip ranges
- `deny` - `Array` the denied ip ranges

Format options:
- `format` - `String` compression method to use: 'short', 'long'

The addresses may be converted from IPv6 to IPv4 and from IPv4 to IPv6 if possible.

Ranges for `deny` and `allow` may contain a list of multiple IP ranges which are
given in with the IP address and the significant bits behind: '127.0.0.1/8' or as
a range name.
Range names are: unspecified, broadcast, multicast, linklocal, loopback, private,
reserved, uniquelocal, ipv4mapped, rfc6145, rfc6052, 6to4, teredo.
Or use the range 'special' to specify all of the named ranges.

|  has allow  |  has deny | in allow | in deny | in both | in other |
|-------------|-----------|----------|---------|---------|----------|
|   no        |   no      |    -     |    -    |    -    |   ok     |
|   yes       |   no      |    ok    |    -    |    -    |   fail   |
|   no        |   yes     |    -     |   fail  |    -    |   ok     |
|   yes       |   yes     |    ok    |   fail  |    ok   |   ok     |

The output will always be without leading '0' and by default compressed to the short
form for IPv6 addresses. To get the long form use the 'format' option.


Schema Specification
---------------------------------------------------
{@schema #selfcheck}
 */

(function() {
  var all, ipaddr, list, name, rules, specialRanges, util;

  ipaddr = require('ipaddr.js');

  util = require('alinex-util');

  rules = require('../helper/rules');

  specialRanges = {
    unspecified: ['0.0.0.0/8', '0::/128'],
    broadcast: ['255.255.255.255/32'],
    multicast: ['224.0.0.0/4', 'ff00::/8'],
    linklocal: ['169.254.0.0/16', 'fe80::/10'],
    loopback: ['127.0.0.0/8', '::1/128'],
    "private": ['10.0.0.0/8', '172.16.0.0/12', '192.168.0.0/16'],
    reserved: ['192.0.0.0/24', '192.88.99.0/24', '198.51.100.0/24', '203.0.113.0/24', '240.0.0.0/4', '2001:db8::/32'],
    uniquelocal: ['fc00::/7'],
    ipv4mapped: ['::ffff:0:0/96'],
    rfc6145: ['::ffff:0:0:0/96'],
    rfc6052: ['64:ff9b::/96'],
    '6to4': ['2002::/16'],
    teredo: ['2001::/32']
  };

  all = [];

  for (name in specialRanges) {
    list = specialRanges[name];
    all.concat(list);
  }

  specialRanges.special = all;

  exports.describe = function(cb) {
    var text;
    text = 'A valid IP address as string. ';
    text += rules.optional.describe.call(this);
    text = text.replace(/\. It's/, ' which is');
    if (this.schema.version) {
      if (this.schema.version === 'ipv4') {
        text += "Only IPv4 addresses are valid. ";
      } else {
        text += "Only IPv6 addresses are valid. ";
      }
      if (this.schema.ipv4Mapping) {
        text += "IPv4 addresses may be automatically converted. ";
      }
    }
    if (this.schema.deny) {
      text += "The IP address should not be in the ranges: '" + (this.schema.deny.join('\', \'')) + "'. ";
      if (this.schema.allow) {
        text += "But IP address in the ranges: '" + (this.schema.allow.join('\', \'')) + "' are allowed. ";
      }
    } else if (this.schema.allow) {
      text += "The IP address have to be in the ranges: '" + (this.schema.allow.join('\', \'')) + "'. ";
    }
    switch (this.schema.format) {
      case 'short':
        text += 'An IPv6 address will be compressed as possible. ';
        break;
      case 'long':
        text += 'An IPv6 address will be normalized with all octets visible. ';
    }
    return cb(null, text);
  };

  exports.check = function(cb) {
    var addr, bits, entry, error, i, ip, j, k, l, len, len1, len2, len3, ref, ref1, ref2, ref3, ref4, ref5, ref6, ref7, skip, subentry;
    skip = rules.optional.check.call(this);
    if (skip instanceof Error) {
      return cb(skip);
    }
    if (skip) {
      return cb();
    }
    try {
      ip = ipaddr.parse(this.value);
    } catch (error1) {
      error = error1;
      if (error) {
        return this.sendError("The given value is no valid IP address", cb);
      }
    }
    if (this.schema.version) {
      if (this.schema.version === 'ipv4') {
        if (ip.kind() === 'ipv6') {
          if (ip.isIPv4MappedAddress() && this.schema.ipv4Mapping) {
            if (this.debug.enabled) {
              this.debug(this.name + ": convert to ipv4");
            }
            ip = ip.toIPv4Address();
          } else {
            return this.sendError("The given value is no valid IPv" + this.schema.version + " address", cb);
          }
        }
      } else {
        if (ip.kind() === 'ipv4') {
          if (!this.schema.ipv4Mapping) {
            return this.sendError("The given value is no valid IPv" + this.schema.version + " address", cb);
          }
          if (this.debug.enabled) {
            this.debug(this.name + ": convert to ipv4mapped");
          }
          ip = ip.toIPv4MappedAddress();
        }
      }
    }
    this.value = ip.kind() === 'ipv6' ? this.schema.format === 'long' ? ip.toNormalizedString() : ip.toString() : ip.toString();
    if (this.schema.allow) {
      ref = this.schema.allow;
      for (i = 0, len = ref.length; i < len; i++) {
        entry = ref[i];
        if (specialRanges[entry] != null) {
          ref1 = specialRanges[entry];
          for (j = 0, len1 = ref1.length; j < len1; j++) {
            subentry = ref1[j];
            ref2 = subentry.split(/\//), addr = ref2[0], bits = ref2[1];
            if (ip.match(ipaddr.parse(addr), bits)) {
              return this.sendSuccess(cb);
            }
          }
        } else {
          ref3 = entry.split(/\//), addr = ref3[0], bits = ref3[1];
          if (ip.match(ipaddr.parse(addr), bits)) {
            return this.sendSuccess(cb);
          }
        }
      }
      if (!this.schema.deny) {
        return this.sendError("The given ip address is not in the allowed ranges", cb);
      }
    }
    if (this.schema.deny) {
      ref4 = this.schema.deny;
      for (k = 0, len2 = ref4.length; k < len2; k++) {
        entry = ref4[k];
        if (specialRanges[entry] != null) {
          ref5 = specialRanges[entry];
          for (l = 0, len3 = ref5.length; l < len3; l++) {
            subentry = ref5[l];
            ref6 = subentry.split(/\//), addr = ref6[0], bits = ref6[1];
            if (ip.match(ipaddr.parse(addr), bits)) {
              return this.sendError("The given ip address is denied because in range " + entry, cb);
            }
          }
        } else {
          ref7 = entry.split(/\//), addr = ref7[0], bits = ref7[1];
          if (ip.match(ipaddr.parse(addr), bits)) {
            return this.sendError("The given ip address is denied because in range " + entry, cb);
          }
        }
      }
    }
    return this.sendSuccess(cb);
  };

  exports.selfcheck = {
    title: "IP Address",
    description: "an ip address schema definition",
    type: 'object',
    allowedKeys: true,
    keys: util.extend({
      version: {
        title: "IP Type",
        description: "the ip address version to use. ",
        type: 'string',
        values: ['ipv4', 'ipv6'],
        optional: true
      },
      ipv4Mapping: {
        title: "Map IPv4 to IPv6",
        description: "the default value to use if nothing given",
        type: 'boolean',
        optional: true
      },
      deny: {
        title: "Deny Addresses",
        description: "a list of addresses or ranges to deny",
        type: 'array',
        optional: true,
        entries: {
          title: "Deny Address",
          description: "the address or range to deny",
          type: 'string',
          match: /^unspecified|broadcast|multicast|linklocal|loopback|private|reserved|uniquelocal|ipv4mapped|rfc(6145|6052)|6to4|teredo|special|(|([0-9a-fA-F]{1,4}:){7,7}[0-9a-fA-F]{1,4}|([0-9a-fA-F]{1,4}:){1,7}:|([0-9a-fA-F]{1,4}:){1,6}:[0-9a-fA-F]{1,4}|([0-9a-fA-F]{1,4}:){1,5}(:[0-9a-fA-F]{1,4}){1,2}|([0-9a-fA-F]{1,4}:){1,4}(:[0-9a-fA-F]{1,4}){1,3}|([0-9a-fA-F]{1,4}:){1,3}(:[0-9a-fA-F]{1,4}){1,4}|([0-9a-fA-F]{1,4}:){1,2}(:[0-9a-fA-F]{1,4}){1,5}|[0-9a-fA-F]{1,4}:((:[0-9a-fA-F]{1,4}){1,6})|:((:[0-9a-fA-F]{1,4}){1,7}|:)|fe80:(:[0-9a-fA-F]{0,4}){0,4}%[0-9a-zA-Z]{1,}|::(ffff(:0{1,4}){0,1}:){0,1}((25[0-5]|(2[0-4]|1{0,1}[0-9]){0,1}[0-9])\.){3,3}(25[0-5]|(2[0-4]|1{0,1}[0-9]){0,1}[0-9])|([0-9a-fA-F]{1,4}:){1,4}:((25[0-5]|(2[0-4]|1{0,1}[0-9]){0,1}[0-9])\.){3,3}(25[0-5]|(2[0-4]|1{0,1}[0-9]){0,1}[0-9]))\/(12[0-8]|(1[01][0-9]){0,1}[0-9])|(((25[0-5]|(2[0-4]|1{0,1}[0-9]){0,1}[0-9])\.){3,3}(25[0-5]|(2[0-4]|1{0,1}[0-9]){0,1}[0-9]))\/(3[0-2]|[12]{0,1}[0-9])$/
        }
      },
      allow: {
        title: "Allow Addresses",
        description: "a list of addresses or ranges to allow",
        type: 'array',
        optional: true,
        entries: {
          title: "Allow Address",
          description: "the address or range to allow",
          type: 'string',
          match: /^unspecified|broadcast|multicast|linklocal|loopback|private|reserved|uniquelocal|ipv4mapped|rfc(6145|6052)|6to4|teredo|special|(|([0-9a-fA-F]{1,4}:){7,7}[0-9a-fA-F]{1,4}|([0-9a-fA-F]{1,4}:){1,7}:|([0-9a-fA-F]{1,4}:){1,6}:[0-9a-fA-F]{1,4}|([0-9a-fA-F]{1,4}:){1,5}(:[0-9a-fA-F]{1,4}){1,2}|([0-9a-fA-F]{1,4}:){1,4}(:[0-9a-fA-F]{1,4}){1,3}|([0-9a-fA-F]{1,4}:){1,3}(:[0-9a-fA-F]{1,4}){1,4}|([0-9a-fA-F]{1,4}:){1,2}(:[0-9a-fA-F]{1,4}){1,5}|[0-9a-fA-F]{1,4}:((:[0-9a-fA-F]{1,4}){1,6})|:((:[0-9a-fA-F]{1,4}){1,7}|:)|fe80:(:[0-9a-fA-F]{0,4}){0,4}%[0-9a-zA-Z]{1,}|::(ffff(:0{1,4}){0,1}:){0,1}((25[0-5]|(2[0-4]|1{0,1}[0-9]){0,1}[0-9])\.){3,3}(25[0-5]|(2[0-4]|1{0,1}[0-9]){0,1}[0-9])|([0-9a-fA-F]{1,4}:){1,4}:((25[0-5]|(2[0-4]|1{0,1}[0-9]){0,1}[0-9])\.){3,3}(25[0-5]|(2[0-4]|1{0,1}[0-9]){0,1}[0-9]))\/(12[0-8]|(1[01][0-9]){0,1}[0-9])|(((25[0-5]|(2[0-4]|1{0,1}[0-9]){0,1}[0-9])\.){3,3}(25[0-5]|(2[0-4]|1{0,1}[0-9]){0,1}[0-9]))\/(3[0-2]|[12]{0,1}[0-9])$/
        }
      },
      format: {
        title: "Format",
        description: "the display format tu use",
        type: 'string',
        "default": 'short',
        values: ['short', 'long']
      }
    }, rules.baseSchema, {
      "default": {
        title: "Default Value",
        description: "the default value to use if nothing given",
        type: 'string',
        match: /^(([0-9a-fA-F]{1,4}:){7,7}[0-9a-fA-F]{1,4}|([0-9a-fA-F]{1,4}:){1,7}:|([0-9a-fA-F]{1,4}:){1,6}:[0-9a-fA-F]{1,4}|([0-9a-fA-F]{1,4}:){1,5}(:[0-9a-fA-F]{1,4}){1,2}|([0-9a-fA-F]{1,4}:){1,4}(:[0-9a-fA-F]{1,4}){1,3}|([0-9a-fA-F]{1,4}:){1,3}(:[0-9a-fA-F]{1,4}){1,4}|([0-9a-fA-F]{1,4}:){1,2}(:[0-9a-fA-F]{1,4}){1,5}|[0-9a-fA-F]{1,4}:((:[0-9a-fA-F]{1,4}){1,6})|:((:[0-9a-fA-F]{1,4}){1,7}|:)|fe80:(:[0-9a-fA-F]{0,4}){0,4}%[0-9a-zA-Z]{1,}|::(ffff(:0{1,4}){0,1}:){0,1}((25[0-5]|(2[0-4]|1{0,1}[0-9]){0,1}[0-9])\.){3,3}(25[0-5]|(2[0-4]|1{0,1}[0-9]){0,1}[0-9])|([0-9a-fA-F]{1,4}:){1,4}:((25[0-5]|(2[0-4]|1{0,1}[0-9]){0,1}[0-9])\.){3,3}(25[0-5]|(2[0-4]|1{0,1}[0-9]){0,1}[0-9]))|(((25[0-5]|(2[0-4]|1{0,1}[0-9]){0,1}[0-9])\.){3,3}(25[0-5]|(2[0-4]|1{0,1}[0-9]){0,1}[0-9]))$/,
        optional: true
      }
    })
  };

}).call(this);

//# sourceMappingURL=ipaddr.map