
/*
Object
=================================================
For all complex data structures you use the object type which checks for named
arrays or instance objects.

This is the most complex validation form because it has different checks and
uses subchecks on each entry.

Sanitize options:
- `flatten` - flatten hierarchical values

Check options:
- `instanceOf` - `Class` only objects of given class type are allowed
- `flatten` - `Boolean` flatten deep structures
- `mandatoryKeys` - `Array|Boolean` the list of elements which are mandatory
- `allowedKeys` - `Array|Boolean` gives a list of elements which are also allowed
  or true to use the list from entries definition

Validating children:
- `entries` - `Object` specification for entries
- `keys` - `Object` specification for all entries per each key name

So you have two different ways to specify objects. First you can use the `instanceOf`
check. Or specify a data object.

The `mandatoryKeys` and `allowedKeys` may both contain normal strings for complete
key names and also regular expressions to match multiple. In case of using it
in the mandatoryKeys field at least one matching key have to be present.
And as you may suspect the `mandatoryKeys` are automatically also `allowedKeys`.
If `mandatoryKeys` or `allowedKeys` are set to true instead of a list all of the
specified keys in entries or keys are meant.

The `keys` specify the subcheck for each containing object attribute. If they are
not optional or contain a default entry they will be seen also as mandatory field.

The `entries` list do the same as the `keys` section but works using key matching
on multiple entires. If an object attribute matches multiple entries-rules the
first will be used.

__Examples:__

The follwoing will check for an instance:

``` coffee
validator.check
  name: 'test'        # name to be displayed in errors (optional)
  value: input        # value to check
  schema:             # definition of checks
    type: 'object'
    instanceOf: RegeExp
, (err, result) ->
   * do something
```

Or you may specify the data object structure:

``` coffee
validator.check
  name: 'test'        # name to be displayed in errors (optional)
  value: input        # value to check
  schema:             # definition of checks
    type: 'object'
    mandatoryKeys: ['name']
    allowedKeys: ['mail', 'phone']
    entries: [
      type: 'string'
    ]
, (err, result) ->
   * do something
```

Here all object values have to be strings.

``` coffee
validator.check
  name: 'test'        # name to be displayed in errors (optional)
  value: input        # value to check
  schema:             # definition of checks
    type: 'object'
    mandatoryKeys: ['name']
    entries: [
      key: /^num-\d+/
      type: 'integer'
    ,
      type: 'string'
    ]
, (err, result) ->
   * do something
```

And here the keys matching the key-check (starting with 'num-...') have to be
integers and all other strings.

If you don't specify `allowedKeys` more attributes with other names are possible.

And the most complex situation is a deep checking structure with checking each
key for its specifics:

``` coffee
validator.check
  name: 'test'        # name to be displayed in errors (optional)
  value: input        # value to check
  schema:             # definition of checks
    type: 'object'
    allowedKeys: true
    keys:
      name:
        type: 'string'
      mail:
        type: 'string'
        optional: true
      phone:
        type: 'string'
        optional: true
, (err, result) ->
   * do something
```

Here `allowedKeys` will check that no attributes are used which are not specified
in the entries. Which attribute is optional may be specified within the attributes
specification. That means this check is the same as above but also checks that the
three attributes are strings.

If you specify `entries` and `keys`, the entries check will only be used as default
for all keys which has no own specification.

Another option is to flatten the structure before checking it:

``` coffee
 * value to check
input =
  first:
    num: { one: 1, two: 2 }
  second:
    num: { one: 1, two: 2 }
    name: { anna: 1, berta: 2 }
 * run the validation
validator.check
  name: 'test'        # name to be displayed in errors (optional)
  value: input        # value to check
  schema:             # definition of checks
    type: 'object'
    flatten: true
, (err, result) ->
   * do something
```

This will give you the following result:

``` coffee
result =
  'first-num': { one: 1, two: 2 }
  'second-num': { one: 1, two: 2 }
  'second-name': { anna: 1, berta: 2 }
```


Optional Elements
----------------------------------------------------
You have two possibilities to make elements optional:

First you can specify the mandatory names and all other are optional:

``` coffee
validator.check
  name: 'test'        # name to be displayed in errors (optional)
  value: input        # value to check
  schema:             # definition of checks
    type: 'object'
    mandatoryKeys: ['name']
    keys:
      name:
        type: 'string'
      age:
        type: `integer`
, (err, result) ->
   * do something
```

Or you can set the generall rule to all are mandatory and define in each one if it
is optional:

``` coffee
validator.check
  name: 'test'        # name to be displayed in errors (optional)
  value: input        # value to check
  schema:             # definition of checks
    type: 'object'
    mandatoryKeys: true
    keys:
      name:
        type: 'string'
      age:
        type: `integer`
        optional: true
, (err, result) ->
   * do something
```


Schema Specification
---------------------------------------------------
{@schema #selfcheck}
 */

(function() {
  var async, flatten, rules, util,
    indexOf = [].indexOf || function(item) { for (var i = 0, l = this.length; i < l; i++) { if (i in this && this[i] === item) return i; } return -1; };

  async = require('async');

  util = require('alinex-util');

  rules = require('../helper/rules');

  exports.describe = function(cb) {
    var allowedKeys, entry, l, len, len1, list, m, mandatoryKeys, ref, ref1, ref2, ref3, text;
    text = 'An object. ';
    text += rules.optional.describe.call(this);
    text = text.replace(/\. It's/, ' which is');
    if (this.schema.flatten) {
      text += "Hierarchical paths will be flattened together. ";
    }
    if (this.schema.instanceOf != null) {
      text += "The object has to be an instance of class `" + this.schema.instanceOf.name + "`. ";
    }
    text = text.replace(/object\. The object/, 'object which');
    mandatoryKeys = (ref = this.schema.mandatoryKeys) != null ? ref : [];
    if (mandatoryKeys && typeof mandatoryKeys === 'boolean') {
      mandatoryKeys = [];
      if (this.schema.keys != null) {
        mandatoryKeys = mandatoryKeys.concat(Object.keys(this.schema.keys));
      }
      if (this.schema.entries != null) {
        ref1 = this.schema.entries;
        for (l = 0, len = ref1.length; l < len; l++) {
          entry = ref1[l];
          if (entry.key != null) {
            mandatoryKeys.push(entry.key);
          }
        }
      }
    }
    if (mandatoryKeys.length) {
      text += "The following keys have to be present: `" + (mandatoryKeys.join('`, `')) + "`. ";
      text = text.replace(/object\. The following/, ' object with the following');
    }
    allowedKeys = (ref2 = this.schema.allowedKeys) != null ? ref2 : [];
    if (allowedKeys && typeof allowedKeys === 'boolean') {
      allowedKeys = [];
      if (this.schema.keys != null) {
        allowedKeys = allowedKeys.concat(Object.keys(this.schema.keys));
      }
      if (this.schema.entries != null) {
        ref3 = this.schema.entries;
        for (m = 0, len1 = ref3.length; m < len1; m++) {
          entry = ref3[m];
          if (entry.key) {
            allowedKeys.push(entry.key);
          }
        }
      }
    }
    if (allowedKeys.length) {
      list = allowedKeys.filter(function(e) {
        return !(indexOf.call(mandatoryKeys, e) >= 0);
      });
      if (list.length) {
        text += "The following keys are allowed: `" + (list.join('`, `')) + "`. ";
        text = text.replace(/object\. The following keys are/, ' object with the following keys');
      }
    }
    return async.parallel([
      (function(_this) {
        return function(cb) {
          var detail;
          if (_this.schema.keys == null) {
            return cb();
          }
          detail = "The following entries have a specific format:";
          return async.map(Object.keys(_this.schema.keys), function(key, cb) {
            var worker;
            worker = _this.sub(_this.name + "." + key, _this.schema.keys[key]);
            return worker.describe(function(err, subtext) {
              if (err) {
                return cb(err);
              }
              return cb(null, "\n\n`" + key + "`\n:   " + (subtext.replace(/\n/g, '\n    ')));
            });
          }, function(err, results) {
            if (err) {
              return cb(err);
            }
            return cb(null, detail + results.join('') + '\n');
          });
        };
      })(this), (function(_this) {
        return function(cb) {
          var detail, o, ref4, results1;
          if (_this.schema.entries == null) {
            return cb();
          }
          if (_this.schema.keys) {
            detail = "And all other keys are: ";
          } else {
            detail = "The entries are: ";
          }
          return async.map((function() {
            results1 = [];
            for (var o = 0, ref4 = _this.schema.entries.length - 1; 0 <= ref4 ? o <= ref4 : o >= ref4; 0 <= ref4 ? o++ : o--){ results1.push(o); }
            return results1;
          }).apply(this), function(num, cb) {
            var rule, ruletext, worker;
            rule = _this.schema.entries[num];
            if (rule.key != null) {
              ruletext = "\n- matching " + rule.key + ": ";
            } else {
              ruletext = "\n- any key: ";
            }
            worker = _this.sub(_this.name + "#" + num, _this.schema.entries[num]);
            return worker.describe(function(err, subtext) {
              if (err) {
                return cb(err);
              }
              return cb(null, ruletext + subtext.replace(/\n/g, '\n  '));
            });
          }, function(err, results) {
            if (err) {
              return cb(err);
            }
            return cb(null, detail + results.join('') + '\n');
          });
        };
      })(this)
    ], function(err, results) {
      if (err) {
        return cb(err);
      }
      return cb(null, (text + results.join('')).trim() + ' ');
    });
  };

  exports.check = function(cb) {
    var allowedKeys, entry, keys, l, len, len1, m, mandatoryKeys, ref, ref1, ref2, ref3, skip, usedKeys;
    skip = rules.optional.check.call(this);
    if (skip instanceof Error) {
      return cb(skip);
    }
    if (skip) {
      return cb();
    }
    if (this.schema.flatten) {
      this.value = flatten(this.value);
    }
    if (this.schema.instanceOf != null) {
      if (!(this.value instanceof this.schema.instanceOf)) {
        return this.sendError("An object of " + this.schema.instanceOf.name + " is needed as value", cb);
      }
    }
    if (typeof this.value !== 'object' || Array.isArray(this.value)) {
      return this.sendError("The value has to be an object", cb);
    }
    usedKeys = Object.keys(this.value);
    mandatoryKeys = (ref = this.schema.mandatoryKeys) != null ? ref : [];
    if (typeof mandatoryKeys === 'boolean') {
      mandatoryKeys = [];
      if (this.schema.keys != null) {
        mandatoryKeys = mandatoryKeys.concat(Object.keys(this.schema.keys));
      }
      if (this.schema.entries != null) {
        ref1 = this.schema.entries;
        for (l = 0, len = ref1.length; l < len; l++) {
          entry = ref1[l];
          if (entry.key != null) {
            mandatoryKeys.push(entry.key);
          }
        }
      }
    }
    allowedKeys = (ref2 = this.schema.allowedKeys) != null ? ref2 : [];
    if (typeof allowedKeys === 'boolean') {
      allowedKeys = [];
      if (this.schema.keys != null) {
        allowedKeys = allowedKeys.concat(Object.keys(this.schema.keys));
      }
      if (this.schema.entries != null) {
        ref3 = this.schema.entries;
        for (m = 0, len1 = ref3.length; m < len1; m++) {
          entry = ref3[m];
          if (entry.key) {
            allowedKeys.push(entry.key);
          }
        }
      }
    }
    keys = util.array.unique(usedKeys.concat(mandatoryKeys, allowedKeys));
    return async.each(keys, (function(_this) {
      return function(key, cb) {
        var i, k, len2, o, ref4, ref5, ref6, ref7, rule, schema, v, worker;
        if (key instanceof RegExp) {
          return cb();
        }
        if ((((ref4 = _this.schema.keys) != null ? ref4[key] : void 0) != null) && typeof _this.schema.keys[key] === 'object') {
          schema = {};
          ref5 = _this.schema.keys[key];
          for (k in ref5) {
            v = ref5[k];
            schema[k] = v;
          }
          if (indexOf.call(mandatoryKeys, key) < 0) {
            if (schema.optional == null) {
              schema.optional = true;
            }
          }
          worker = _this.sub(_this.name + "." + key, schema, _this.value[key]);
        } else if (_this.schema.entries != null) {
          ref6 = _this.schema.entries;
          for (i = o = 0, len2 = ref6.length; o < len2; i = ++o) {
            rule = ref6[i];
            schema = {};
            ref7 = _this.schema.entries[i];
            for (k in ref7) {
              v = ref7[k];
              schema[k] = v;
            }
            if (indexOf.call(mandatoryKeys, key) < 0) {
              if (schema.optional == null) {
                schema.optional = true;
              }
            }
            if (rule.key != null) {
              if (!key.match(rule.key)) {
                continue;
              }
              worker = _this.sub(_this.name + "#entries-" + i + "." + key, schema, _this.value[key]);
              break;
            } else {
              worker = _this.sub(_this.name + "#entries-" + i + "." + key, schema, _this.value[key]);
            }
          }
        }
        if (!worker) {
          worker = _this.sub(_this.name + "#." + key, {
            type: (function() {
              switch (false) {
                case !Array.isArray(this.value[key]):
                  return 'array';
                case typeof this.value[key] !== 'object':
                  return 'object';
                default:
                  return 'any';
              }
            }).call(_this),
            optional: true
          }, _this.value[key]);
        }
        return async.setImmediate(function() {
          return worker.check(function(err) {
            if (err) {
              return cb(err);
            }
            if (worker.value) {
              _this.value[key] = worker.value;
            } else {
              delete _this.value[key];
            }
            return cb();
          });
        });
      };
    })(this), (function(_this) {
      return function(err) {
        var allow, fail, isAllowed, key, len2, len3, len4, mandatory, o, p, q, ref4, ref5;
        if (err) {
          return cb(err);
        }
        for (o = 0, len2 = mandatoryKeys.length; o < len2; o++) {
          mandatory = mandatoryKeys[o];
          if (mandatory instanceof RegExp) {
            fail = true;
            for (key in _this.value) {
              if (key.match(mandatory)) {
                fail = false;
                break;
              }
            }
            if (fail) {
              return _this.sendError("The mandatory key '" + key + "' is missing", cb);
            }
          } else {
            if (!((_this.value[mandatory] != null) || ((ref4 = _this.schema.keys) != null ? (ref5 = ref4[mandatory]) != null ? ref5.optional : void 0 : void 0))) {
              return _this.sendError("The mandatory key '" + mandatory + "' is missing", cb);
            }
          }
        }
        if (allowedKeys.length) {
          for (key in _this.value) {
            isAllowed = false;
            for (p = 0, len3 = allowedKeys.length; p < len3; p++) {
              allow = allowedKeys[p];
              if (key === allow || (allow instanceof RegExp && key.match(allow))) {
                isAllowed = true;
                break;
              }
            }
            if (!isAllowed) {
              for (q = 0, len4 = mandatoryKeys.length; q < len4; q++) {
                allow = mandatoryKeys[q];
                if (key === allow || (allow instanceof RegExp && key.match(allow))) {
                  isAllowed = true;
                  break;
                }
              }
            }
            if (!isAllowed) {
              return _this.sendError("The key '" + key + "' is not allowed", cb);
            }
          }
        }
        return _this.sendSuccess(cb);
      };
    })(this));
  };

  exports.selfcheck = {
    title: "Object",
    description: "the object schema definitions",
    type: 'object',
    allowedKeys: true,
    keys: util.extend({
      flatten: {
        title: "Flatten",
        description: "a flag to flatten the object structure",
        type: 'boolean',
        optional: true
      },
      instanceOf: {
        title: "Class Check",
        description: "the class, the object to be instantiated from",
        type: 'function',
        optional: true
      },
      mandatoryKeys: {
        title: "Mandatory Keys",
        description: "the definition of mandatory keys",
        type: 'or',
        optional: true,
        or: [
          {
            title: "All Mandatory",
            description: "a flag if set to `true` marks all schema defined keys mandatory",
            type: 'boolean'
          }, {
            title: "Mandatory List",
            description: "the list of mandatory keys",
            type: 'array',
            entries: {
              title: "Mandatory Key",
              description: "the key which to be present",
              type: 'or',
              or: [
                {
                  title: "Key Name",
                  description: "the name of the mandatory key",
                  type: 'string'
                }, {
                  title: "Key Map",
                  description: "a RegExp to detect mandatory keys to which at least one should match",
                  type: 'regexp'
                }
              ]
            }
          }
        ]
      },
      allowedKeys: {
        title: "Allowed Keys",
        description: "the definition of allowed keys",
        type: 'or',
        optional: true,
        or: [
          {
            title: "No more Allowed",
            description: "a flag if set to `true` marks only the schema defined keys as allowed",
            type: 'boolean'
          }, {
            title: "Allowed List",
            description: "the list of allowed keys",
            type: 'array',
            entries: {
              title: "Allowed Key",
              description: "the key which may be present",
              type: 'or',
              or: [
                {
                  title: "Key Name",
                  description: "the name of the allowed key",
                  type: 'string'
                }, {
                  title: "Key Map",
                  description: "a RegExp to detect allowed keys to which should match",
                  type: 'regexp'
                }
              ]
            }
          }
        ]
      },
      entries: {
        title: "Entries",
        description: "an alternative definition of key's types without the name",
        type: 'array',
        mandatoryKeys: ['type'],
        optional: true
      },
      keys: {
        title: "Keys",
        description: "the definition of each key's types",
        type: 'object',
        optional: true
      }
    }, rules.baseSchema, {
      "default": {
        title: "Default Value",
        description: "the default value to use if nothing given",
        type: 'object',
        optional: true
      }
    })
  };

  flatten = function(obj) {
    var j, k, n, v, w;
    n = {};
    for (k in obj) {
      v = obj[k];
      if (typeof v !== 'object') {
        return obj;
      }
      for (j in v) {
        w = v[j];
        if (typeof w !== 'object') {
          return obj;
        }
      }
      v = flatten(v);
      for (j in v) {
        w = v[j];
        n[k + "-" + j] = w;
      }
    }
    return n;
  };

}).call(this);

//# sourceMappingURL=object.map