
/*
Or
=================================================
This is used to give some alternatives from which at least one check have to
succeed. The first one succeeding will work.

__Option:__

- `or` - (array) with different check alternatives

__Example:__

You may allow numeric and special format input:

``` coffee
validator.check
  name: 'test'        # name to be displayed in errors (optional)
  value: input        # value to check
  schema:             # definition of checks
    type: 'or'
    or: [
      type: 'float'
    ,
      type: 'string'
      match: ///
        ^\s*      # start with possible spaces
        [+-]?     # sign possible
        \s*\d+(\.\d*)? # float number
        \s*%?     # percent sign with spaces
        \s*$      # end of text with spaces
        ///
    ]
, (err, result) ->
   * do something
```

With this type you can also use different option alternatives:

``` coffee
validator.check
  name: 'test'        # name to be displayed in errors (optional)
  value: input        # value to check
  schema:             # definition of checks
    type: 'or'
    or: [
      type: 'object'
      allowedKeys: true
      keys:
        type:
          type: 'string'
          lowerCase: true
          values: ['mysql']
        port:
          type: 'integer'
          default: 3306
         * ...
    ,
      type: 'object'
      allowedKeys: true
      keys:
        type:
          type: 'string'
          lowerCase: true
          values: ['postgres']
        port:
          type: 'integer'
          default: 5432
         * ...
    ]
, (err, result) ->
   * do something
```

In the example above only the default port is changed, but you may also add different
options.


Schema Specification
---------------------------------------------------
{@schema #selfcheck}
 */

(function() {
  var async, rules, util;

  async = require('async');

  util = require('alinex-util');

  rules = require('../helper/rules');

  exports.describe = function(cb) {
    var i, ref, results1, text;
    text = "It has to be __one of__ the following types: ";
    text += rules.optional.describe.call(this);
    text = text.replace(/: It's optional./, ' (optional):');
    return async.map((function() {
      results1 = [];
      for (var i = 0, ref = this.schema.or.length - 1; 0 <= ref ? i <= ref : i >= ref; 0 <= ref ? i++ : i--){ results1.push(i); }
      return results1;
    }).apply(this), (function(_this) {
      return function(num, cb) {
        var worker;
        worker = _this.sub(_this.name + "#" + num, _this.schema.or[num]);
        return worker.describe(function(err, subtext) {
          if (err) {
            return cb(err);
          }
          return cb(null, "\n- " + (subtext.replace(/\n/g, '\n  ')));
        });
      };
    })(this), function(err, results) {
      if (err) {
        return cb(err);
      }
      text += results.join('') + '\n';
      return cb(null, text);
    });
  };

  exports.check = function(cb) {
    var error, i, ref, results1, skip;
    skip = rules.optional.check.call(this);
    if (skip instanceof Error) {
      return cb(skip);
    }
    if (skip) {
      return cb();
    }
    error = [];
    return async.map((function() {
      results1 = [];
      for (var i = 0, ref = this.schema.or.length - 1; 0 <= ref ? i <= ref : i >= ref; 0 <= ref ? i++ : i--){ results1.push(i); }
      return results1;
    }).apply(this), (function(_this) {
      return function(num, cb) {
        var worker;
        worker = _this.sub(_this.name + "#" + num, _this.schema.or[num], util.clone(_this.value));
        return worker.check(function(err) {
          if (err) {
            error[num] = err;
            return cb();
          }
          return cb(null, worker.value);
        });
      };
    })(this), (function(_this) {
      return function(err, results) {
        var j, len, result;
        for (j = 0, len = results.length; j < len; j++) {
          result = results[j];
          if (result == null) {
            continue;
          }
          _this.value = result;
          return _this.sendSuccess(cb);
        }
        return _this.sendError("None of the alternatives are matched (" + (error.map(function(e) {
          return e.message;
        }).join('/ ').trim()) + ")", cb);
      };
    })(this));
  };

  exports.selfcheck = {
    title: "Or",
    description: "alternative schema definitions",
    type: 'object',
    allowedKeys: true,
    keys: util.extend({
      or: {
        title: "Alternatives",
        description: "the list of alternatives for the value",
        type: 'array',
        list: {
          title: "Alternative",
          description: "an alternative for the value",
          type: 'object',
          mandatoryKeys: ['type']
        }
      }
    }, rules.baseSchema, {
      "default": {
        title: "Default Value",
        description: "the default value to use if nothing given",
        type: 'any',
        optional: true
      }
    })
  };

}).call(this);

//# sourceMappingURL=or.map