
/*
Percent
=================================================
Number validation like float but specified for percent and other fraction numbers
like per mille...

Sanitize options allowed:
- `sanitize` - `Boolean` remove invalid characters
- `unit` - `String` unit to convert to if no number is given
- `round` - `Boolean|String` rounding of float can be set to true for arithmetic rounding
  or use `floor` or `ceil` for the corresponding methods
- `decimals` - `Integer` number of decimal digits to round to

Check options:
- `min` - `Numeric` the smallest allowed number
- `max` - `Numeric` the biggest allowed number

Format options:
- `toUnit` - `String` unit to convert value to
- `format` - `String` format number as string
- 'locale' - `String` locale format to use for string output

#3 Possible Units

- '%' - percent value - 10^-2^ = 1 part in 100
- '‰' - per mille value - 10^-3^ = 1 part in 1000
- '‱', 'bp' - basis point - 10^-4^ = 1 part in 10000
- 'pcm' - per cent mille - 10^-5^ = 1 part in 100000
- 'ppm' - parts per million - 10^-6^
- 'ppb' - parts per billion - 10^-9^
- 'ppt' - parts per trillion - 10^-12^
- 'ppq' - parts per quadrillion - 10^-15^


Schema Specification
---------------------------------------------------
{@schema #selfcheck}
 */

(function() {
  var rules, util;

  util = require('alinex-util');

  rules = require('../helper/rules');

  exports.init = function() {
    if (this.schema.decimals && (this.schema.round == null)) {
      this.schema.round = true;
    }
    if (this.schema.round && (this.schema.decimals == null)) {
      return this.schema.decimals = 0;
    }
  };

  exports.describe = function(cb) {
    var text, worker;
    text = 'A percentage value as decimal like 0.3 but it may be given as percent or per mille value text like 30% or 30‰, too. ';
    text += rules.optional.describe.call(this);
    text = text.replace(/\. It's/, ' which is');
    if (this.schema.unit) {
      text += "If no other unit given the number will be interpreted as " + this.schema.unit + ". ";
    }
    if (this.schema.toUnit) {
      text += "The value will be returned as " + this.schema.toUnit + ". ";
    }
    worker = this.sub(this.name + "#", {
      type: 'float',
      round: this.schema.round,
      decimals: this.schema.decimals,
      min: this.schema.min,
      max: this.schema.max,
      format: this.schema.format,
      locale: this.schema.locale
    });
    return worker.describe(function(err, subtext) {
      if (err) {
        return cb(err);
      }
      return cb(null, text + subtext);
    });
  };

  exports.check = function(cb) {
    var match, skip, value, worker;
    skip = rules.optional.check.call(this);
    if (skip instanceof Error) {
      return cb(skip);
    }
    if (skip) {
      return cb();
    }
    if (this.schema.unit != null) {
      if (typeof this.value === 'number' || (typeof this.value === 'string' && this.value.match(/^\s*[+-]?\s*\d+(\.\d*)?\s*$/))) {
        this.value = "" + this.value + this.schema.unit;
      }
    }
    if (typeof this.value === 'string') {
      value = this.value;
      value = this.schema.sanitize ? value.replace(/^.*?([-+]?\d+\.?\d*).*?$/, '$1') : value.replace(/(\d+)[^0-9]*?$/, '$1');
      value = Number(value);
      if (match = this.value.match(/^(.*?)([%‰]|bp|pcm|pp[mbtq])$/)) {
        switch (match[2]) {
          case '%':
            value /= 100;
            break;
          case '‰':
            value /= 1000;
            break;
          case '‱':
          case 'bp':
            value /= 10000;
            break;
          case 'pcm':
            value /= 100000;
            break;
          case 'ppm':
            value /= 1000000;
            break;
          case 'ppb':
            value /= 1000000000;
            break;
          case 'ppt':
            value /= 1000000000000;
            break;
          case 'ppq':
            value /= 1000000000000000;
        }
      } else if (!this.value.match(/\d\s*$/)) {
        return this.sendError("The value can't be parsed, maybe an unknown unit was used", cb);
      }
      this.value = value;
    }
    if (this.schema.toUnit) {
      switch (this.schema.toUnit) {
        case '%':
          this.value *= 100;
          break;
        case '‰':
          this.value *= 1000;
          break;
        case '‱':
        case 'bp':
          this.value *= 10000;
          break;
        case 'pcm':
          this.value *= 100000;
          break;
        case 'ppm':
          this.value *= 1000000;
          break;
        case 'ppb':
          this.value *= 1000000000;
          break;
        case 'ppt':
          this.value *= 1000000000000;
          break;
        case 'ppq':
          this.value *= 1000000000000000;
      }
    }
    worker = this.sub(this.name + "#", {
      type: 'float',
      round: this.schema.round,
      decimals: this.schema.decimals,
      min: this.schema.min,
      max: this.schema.max,
      format: this.schema.format,
      locale: this.schema.locale
    }, this.value);
    return worker.check((function(_this) {
      return function(err) {
        if (err) {
          return cb(err);
        }
        _this.value = worker.value;
        return _this.sendSuccess(cb);
      };
    })(this));
  };

  exports.selfcheck = {
    title: "Percent",
    description: "a percent schema definition",
    type: 'object',
    allowedKeys: true,
    keys: util.extend({
      sanitize: {
        title: "Sanitize",
        description: "a flag which allows removing of non numeric characters before evaluating",
        type: 'boolean',
        optional: true
      },
      unit: {
        title: "Source Unit",
        description: "the unit in which an only numeric value is given, will transform to base value",
        type: 'string',
        optional: true,
        values: ['%', '‰', '‱', 'bp', 'pcm', 'ppm', 'ppb', 'ppt', 'ppq']
      },
      round: {
        title: "Rounding",
        description: "the value can be rounded in different ways",
        type: 'or',
        optional: true,
        or: [
          {
            title: "Arithmetic Rounding",
            description: "a flag which allows arithmetic rounding (till 4 down, from 5 up) if set to `true`",
            type: 'boolean'
          }, {
            title: "Alternative Rounding",
            description: "the alternative rounding method to be used",
            type: 'string',
            values: ['floor', 'ceil']
          }
        ]
      },
      decimals: {
        title: "Decimals",
        description: "the number of decimal digits to round to",
        type: 'integer',
        optional: true,
        min: 0
      },
      min: {
        title: "Min Value",
        description: "the minimal value to be set",
        type: 'float',
        optional: true
      },
      max: {
        title: "Max Value",
        description: "the maximal value to be set",
        type: 'float',
        optional: true,
        min: '<<<min>>>'
      },
      toUnit: {
        title: "Result Unit",
        description: "the unit to which to transform the value",
        type: 'string',
        values: ['%', '‰', '‱', 'bp', 'pcm', 'ppm', 'ppb', 'ppt', 'ppq'],
        optional: true
      },
      format: {
        title: "Format",
        description: "the numerical output format to use",
        type: 'string',
        optional: true
      },
      locale: {
        title: "Locale",
        description: "the language to be used for locale specific number format",
        type: 'string',
        match: /^[a-z]{2}(?:-[A-Z]{2})?$/,
        optional: true
      }
    }, rules.baseSchema, {
      "default": {
        title: "Default Value",
        description: "the default value to use if nothing given",
        type: 'float',
        optional: true
      }
    })
  };

}).call(this);

//# sourceMappingURL=percent.map