
/*
String
=================================================
This will test for strings and have lots of sanitize and optimization filters
and also different check settings to use.

Sanitize options allowed:
- `makeString` - `Boolean` convert objects to string, first
- `allowControls` - `Boolean` keep control characters in string instead of
  stripping them (but keep \\r\\n)
- `stripTags` - `Boolean` remove all html tags
- `lowerCase` - `Boolean|String` set to `true` or `first`
- `upperCase` - `Boolean|String` set to `true` or `first`
- `replace` - `Array` replacements: string (only single replacement) or
  regular expressions and replacements as inner array
- `trim` - `Boolean` strip whitespace from the beginning and end
- `crop` - `Integer` crop text after number of characters

Check options:
- `optional` - `Boolean` the value must not be present (will return null)
- `minLength` - `Integer` minimum text length in characters
- `maxLength` - `Integer` maximum text length in characters
- `values` - `Array|Object|String` array of possible values (complete text)
- `startsWith` - `String` start of text
- `endsWith` - `String` end of text
- `match` - `String|RegExp` string or regular expression which have to be matched
  (or list of expressions)
- `matchNot` - `String|RegExp` string or regular expression which is not allowed to
  match (or list of expressions)

__Example:__

``` coffee
validator.check
  name: 'test'        # name to be displayed in errors (optional)
  value: input        # value to check
  schema:             # definition of checks
    type: 'string'
    lowerCase: true
    upperCase: 'first'
    values: ['One', 'Two', 'Three']
, (err, result) ->
   * do something
```

#3 Character Case

Instead of setting to `lowerCase` or `upperCase` you can also set both. If you
set one to `true` and the other to `first` you can make all lowercase but first
character uppercase or the other way.


Schema Specification
---------------------------------------------------
{@schema #selfcheck}
 */

(function() {
  var rules, util,
    indexOf = [].indexOf || function(item) { for (var i = 0, l = this.length; i < l; i++) { if (i in this && this[i] === item) return i; } return -1; };

  util = require('alinex-util');

  rules = require('../helper/rules');

  exports.describe = function(cb) {
    var entry, i, j, k, len, len1, len2, list, pattern, ref, ref1, ref2, ref3, ref4, remove, replace, text;
    text = 'A text entry. ';
    text += rules.optional.describe.call(this);
    text = text.replace(/\. It's/, ' which is');
    if ((this.schema.makeString != null) && this.schema.makeString === true) {
      text += "Objects will be converted to their string representation. ";
    }
    remove = [];
    if (!this.schema.allowControls) {
      remove.push("control characters");
    }
    if (this.schema.stripTags) {
      remove.push("HTML Tags");
    }
    if (remove.length) {
      text += "All " + (remove.join(', ')) + " will be removed. ";
    }
    text = text.replace(/text entry\. All/, 'text entry in which all');
    if ((this.schema.lowerCase != null) && this.schema.lowerCase === true) {
      text += "The text will get lower case. ";
    }
    if ((this.schema.upperCase != null) && this.schema.upperCase === true) {
      text += "The text will get upper case. ";
    }
    if ((this.schema.lowerCase != null) && this.schema.lowerCase === 'first') {
      text += "The first letter will get lower case. ";
    }
    if ((this.schema.upperCase != null) && this.schema.upperCase === 'first') {
      text += "The first letter will get upper case. ";
    }
    if (this.schema.replace != null) {
      text += "The following replacements will take place: ";
      if (Array.isArray(this.schema.replace[0])) {
        ref = this.schema.replace;
        for (i = 0, len = ref.length; i < len; i++) {
          ref1 = ref[i], pattern = ref1[0], replace = ref1[1];
          text += pattern + " => '" + replace + "', ";
        }
        text = text.replace(/, $/, '. ');
      } else {
        ref2 = this.schema.replace, pattern = ref2[0], replace = ref2[1];
        text += pattern + " => '" + replace + "'. ";
      }
    }
    if (this.schema.trim) {
      text += "Whitespace will be removed at start and end of the text. ";
    }
    if (this.schema.crop != null) {
      text += "The text will be cropped after " + this.schema.crop + " characters. ";
    }
    if ((this.schema.minLength != null) && (this.schema.maxLength != null)) {
      text += "It has to be between " + this.schema.minLength + " and " + this.schema.maxLength + " characters long. ";
    } else if (this.schema.minLength != null) {
      text += "It has to be at least " + this.schema.minLength + " characters long. ";
    } else if (this.schema.maxLength != null) {
      text += "It has to be not more than " + this.schema.maxLength + " characters long. ";
    }
    if (list = this.schema.values) {
      if (typeof list === 'string') {
        list = list.split(/,\s*/);
      } else if (typeof list === 'object' && !Array.isArray(list)) {
        list = Object.keys(list);
      }
      text += "Only the values: '" + (list.join("', '")) + "' are allowed. ";
    }
    if (this.schema.startsWith != null) {
      text += "It has to start with " + this.schema.startsWith + "... ";
    }
    if (this.schema.endsWith != null) {
      text += "It has to end with ..." + this.schema.endsWith + ". ";
    }
    if (this.schema.match != null) {
      text += "The text should match: ";
      if (Array.isArray(this.schema.match)) {
        ref3 = this.schema.match;
        for (j = 0, len1 = ref3.length; j < len1; j++) {
          entry = ref3[j];
          text += entry + ", ";
        }
        text = text.replace(/, $/, '. ');
      } else {
        text += this.schema.match + ". ";
      }
    }
    if (this.schema.matchNot != null) {
      text += "The text shouldn't match: ";
      if (Array.isArray(this.schema.matchNot)) {
        ref4 = this.schema.matchNot;
        for (k = 0, len2 = ref4.length; k < len2; k++) {
          entry = ref4[k];
          text += entry + ", ";
        }
        text = text.replace(/, $/, '. ');
      } else {
        text += this.schema.matchNot + ". ";
      }
    }
    return cb(null, text);
  };

  exports.check = function(cb) {
    var i, j, k, len, len1, len2, list, match, pattern, ref, ref1, ref2, ref3, ref4, ref5, replace, skip, success;
    skip = rules.optional.check.call(this);
    if (skip instanceof Error) {
      return cb(skip);
    }
    if (skip) {
      return cb();
    }
    if (this.schema.makeString) {
      this.value = this.value.toString();
    }
    if (typeof this.value !== 'string') {
      return this.sendError("A string is needed but got " + (typeof this.value) + " instead", cb);
    }
    if (!this.schema.allowControls) {
      this.value = this.value.replace(/[\x00-\x09\x0B\x0C\x0E-\x1F\x7F]/g, '');
    }
    if (this.schema.stripTags) {
      this.value = this.value.replace(/<\/?[^>]+(>|$)/g, '');
    }
    if ((this.schema.lowerCase != null) && this.schema.lowerCase === true) {
      this.value = this.value.toLowerCase();
    }
    if ((this.schema.upperCase != null) && this.schema.upperCase === true) {
      this.value = this.value.toUpperCase();
    }
    if ((this.schema.lowerCase != null) && this.schema.lowerCase === 'first') {
      this.value = this.value.charAt(0).toLowerCase() + this.value.slice(1);
    }
    if ((this.schema.upperCase != null) && this.schema.upperCase === 'first') {
      this.value = this.value.charAt(0).toUpperCase() + this.value.slice(1);
    }
    if (this.schema.replace != null) {
      if (Array.isArray(this.schema.replace[0])) {
        ref = this.schema.replace;
        for (i = 0, len = ref.length; i < len; i++) {
          ref1 = ref[i], pattern = ref1[0], replace = ref1[1];
          this.value = this.value.replace(pattern, replace);
        }
      } else {
        ref2 = this.schema.replace, pattern = ref2[0], replace = ref2[1];
        this.value = this.value.replace(pattern, replace);
      }
    }
    if (this.schema.trim) {
      this.value = this.value.trim();
    }
    if (this.schema.crop != null) {
      this.value = this.value.substring(0, this.schema.crop);
    }
    if ((this.schema.minLength != null) && this.value.length < this.schema.minLength) {
      return this.sendError("The given string '" + this.value + "' is too short at least " + this.schema.minLength + " characters are needed", cb);
    }
    if ((this.schema.maxLength != null) && this.value.length > this.schema.maxLength) {
      return this.sendError("The given string is too long, not more than " + this.schema.maxLength + " characters are allowed", cb);
    }
    if (list = this.schema.values) {
      if (typeof list === 'string') {
        list = list.split(/,\s*/);
      } else if (typeof list === 'object' && !Array.isArray(list)) {
        list = Object.keys(list);
      }
      if (!(Array.isArray(list) && (ref3 = this.value, indexOf.call(list, ref3) >= 0))) {
        return this.sendError("The given string is not in the list of allowed phrases (" + list + ")", cb);
      }
    }
    if ((this.schema.startsWith != null) && this.value.slice(0, +(this.schema.startsWith.length - 1) + 1 || 9e9) !== this.schema.startsWith) {
      return this.sendError("The given string should start with '" + this.schema.startsWith + "'", cb);
    }
    if ((this.schema.endsWith != null) && this.value.slice(this.value.length - this.schema.endsWith.length) !== this.schema.endsWith) {
      return this.sendError("The given string should end with '" + this.schema.endsWith + "'", cb);
    }
    if (this.schema.match != null) {
      if (Array.isArray(this.schema.match)) {
        success = true;
        ref4 = this.schema.match;
        for (j = 0, len1 = ref4.length; j < len1; j++) {
          match = ref4[j];
          if (!(match instanceof RegExp)) {
            match = new RegExp(match);
          }
          success = success && this.value.match(match);
        }
        if (!success) {
          return this.sendError("The given string should match against '" + this.schema.match + "'", cb);
        }
      } else if (this.schema.match instanceof RegExp) {
        if (!this.value.match(this.schema.match)) {
          return this.sendError("The given string should match against '" + this.schema.match + "'", cb);
        }
      } else if (!~this.value.indexOf(this.schema.match)) {
        return this.sendError("The given string should contain '" + this.schema.match + "'", cb);
      }
    }
    if (this.schema.matchNot != null) {
      if (Array.isArray(this.schema.matchNot)) {
        success = true;
        ref5 = this.schema.matchNot;
        for (k = 0, len2 = ref5.length; k < len2; k++) {
          match = ref5[k];
          if (!(match instanceof RegExp)) {
            match = new RegExp(match);
          }
          success = success && !this.value.match(match);
        }
        if (!success) {
          return this.sendError("The given strinG shouldn't match against '" + this.schema.match + "'", cb);
        }
      } else if (this.schema.matchNot instanceof RegExp) {
        if (this.value.match(this.schema.matchNot)) {
          return this.sendError("The given string shouldn't match against '" + this.schema.matchNot + "'", cb);
        }
      } else if (~this.value.indexOf(this.schema.matchNot)) {
        return this.sendError("The given string shouldn't contain '" + this.schema.matchNot + "'", cb);
      }
    }
    return this.sendSuccess(cb);
  };

  exports.selfcheck = {
    title: "String",
    description: "a string schema definition",
    type: 'object',
    allowedKeys: true,
    keys: util.extend({
      makeString: {
        title: "Make String",
        description: "a switch to transform objects into string using the `toString()` method",
        type: 'boolean',
        optional: true
      },
      allowControls: {
        title: "Allow Controls",
        description: "a flag defining that controls are allowed if set to `true`",
        type: 'boolean',
        optional: true
      },
      stripTags: {
        title: "Strip Tags",
        description: "a flag defining if all MTML tags should be removed",
        type: 'boolean',
        optional: true
      },
      lowerCase: {
        title: "Lower Case",
        description: "the flag to transform first or all characters to lower case",
        type: 'or',
        optional: true,
        or: [
          {
            title: "Lower Case All",
            description: "the flag to transform all characters to lower case if set to `true`",
            type: 'boolean'
          }, {
            title: "Lower Case First",
            description: "a flag if set to `first` it will transform the first character to lower case",
            type: 'string',
            values: ['first']
          }
        ]
      },
      upperCase: {
        title: "Upper Case",
        description: "the  flag to transform first or all characters to upper case",
        type: 'or',
        optional: true,
        or: [
          {
            title: "Upper Case All",
            description: "the flag to transform all characters to upper case if set to `true`",
            type: 'boolean'
          }, {
            title: "Lower Case First",
            description: "a flag if set to `first` it will transform the first character to upper case",
            type: 'string',
            values: ['first']
          }
        ]
      },
      replace: {
        title: "Replacement",
        description: "a part to be replaced",
        type: 'or',
        optional: true,
        or: [
          {
            title: "One Replacement",
            description: "the replacement to be done",
            type: 'array',
            list: {
              0: {
                title: "RegExp Match",
                description: "the regular expression to be matched",
                type: 'object',
                instanceOf: RegExp
              },
              1: {
                title: "Replacement String",
                description: "the replacement for each match",
                type: 'string'
              }
            }
          }, {
            title: "Multiple Replacements",
            description: "a list of replacements to be done",
            type: 'array',
            entries: {
              title: "One Replacement",
              description: "the replacement to be done",
              type: 'array',
              list: {
                0: {
                  title: "RegExp Match",
                  description: "the regular expression to be matched",
                  type: 'object',
                  instanceOf: RegExp
                },
                1: {
                  title: "Replacement String",
                  description: "the replacement for each match",
                  type: 'string'
                }
              }
            }
          }
        ]
      },
      trim: {
        title: "Trim",
        description: "a flag set to `true` to trim whitespace from the start and end",
        type: 'boolean',
        optional: true
      },
      crop: {
        title: "Crop Length",
        description: "the maximum text length, if greater it will be cut",
        type: 'integer',
        optional: true,
        min: 1
      },
      minLength: {
        title: "Minimum Length",
        description: "the minimum character length of the text",
        type: 'integer',
        optional: true,
        min: 0
      },
      maxLength: {
        title: "Maximum Length",
        description: "the maximum character length of the text",
        type: 'integer',
        optional: true,
        min: '<<<min>>>'
      },
      values: {
        title: "Value List",
        description: "the list of possible values for this element",
        type: 'or',
        optional: true,
        or: [
          {
            title: "List of Values",
            description: "the list of all possible values",
            type: 'array',
            minLength: 1,
            entries: {
              title: "Possible Value",
              description: "a possible value",
              type: 'string'
            }
          }, {
            title: "Object of Values",
            description: "an object from which one of the keys have to be set as value",
            type: 'object',
            minLength: 1
          }, {
            title: "Comma List of Values",
            description: "a comma separated list of possible values",
            type: 'string'
          }
        ]
      },
      startsWith: {
        title: "Starts With",
        description: "the text has to start with the given phrase",
        type: 'string',
        optional: true
      },
      endsWith: {
        title: "Ends With",
        description: "the text has to end with the given phrase",
        type: 'string',
        optional: true
      },
      match: {
        title: "Matches",
        description: "the text have to match this regular expression",
        type: 'or',
        optional: true,
        or: [
          {
            title: "List of Matches",
            description: "a list of matches to succeed",
            type: 'array',
            entries: {
              title: "One Match",
              description: "the match to succeed",
              type: 'or',
              or: [
                {
                  title: "RegExp String",
                  description: "the match to be checked for success",
                  type: 'string'
                }, {
                  title: "RegExp String",
                  description: "the match to be checked for success",
                  type: 'object',
                  instanceOf: RegExp
                }
              ]
            }
          }, {
            title: "One Match",
            description: "the match to succeed",
            type: 'or',
            or: [
              {
                title: "RegExp String",
                description: "the match to be checked for success",
                type: 'string'
              }, {
                title: "RegExp Object",
                description: "the match to be checked for success",
                type: 'object',
                instanceOf: RegExp
              }
            ]
          }
        ]
      },
      matchNot: {
        title: "Negative Matches",
        description: "the text should not match this regular expression",
        type: 'or',
        optional: true,
        or: [
          {
            title: "List of Negative Matches",
            description: "a list of matches which should not succeed",
            type: 'array',
            entries: {
              title: "One Negative Match",
              description: "the match which should not succeed",
              type: 'or',
              or: [
                {
                  title: "RegExp String",
                  description: "the match to be checked for failure",
                  type: 'string'
                }, {
                  title: "RegExp String",
                  description: "the match to be checked for failure",
                  type: 'object',
                  instanceOf: RegExp
                }
              ]
            }
          }, {
            title: "One Match",
            description: "the match which should not succeed",
            type: 'or',
            or: [
              {
                title: "RegExp String",
                description: "the match to be checked for failure",
                type: 'string'
              }, {
                title: "RegExp Object",
                description: "the match to be checked for failure",
                type: 'object',
                instanceOf: RegExp
              }
            ]
          }
        ]
      }
    }, rules.baseSchema, {
      "default": {
        title: "Default Value",
        description: "the default value to use if nothing given",
        type: 'string',
        optional: true
      }
    })
  };

}).call(this);

//# sourceMappingURL=string.map