
/*
URL
=================================================
Checking text for a valid URL.

Sanitize options:
- `toAbsoluteBase` - `String` convert to absolute with given base
- `removeQuery` - `Boolean` remove query and hash from url

Check options:
- `hostsAllowed` - `String|RegExp` list of allowed hosts by string or regexp
- `hostsDenied` - `String|RegExp` list of denied hosts by string or regexp
- `allowProtocols` - `Array` lust of allowed protocols
- `allowRelative` - `Boolean` to allow also relative urls


Schema Specification
---------------------------------------------------
{@schema #selfcheck}
 */

(function() {
  var rules, url, util,
    indexOf = [].indexOf || function(item) { for (var i = 0, l = this.length; i < l; i++) { if (i in this && this[i] === item) return i; } return -1; };

  url = require('url');

  util = require('alinex-util');

  rules = require('../helper/rules');

  exports.describe = function(cb) {
    var text;
    text = 'A valid url (unified resource locator). ';
    text += rules.optional.describe.call(this);
    text = text.replace(/\. It's/, ' which is');
    if (this.schema.toAbsoluteBase) {
      text += "It will be made absolute from '" + this.schema.toAbsoluteBase + "'. ";
    }
    if (this.schema.removeQuery) {
      text += "Existing query parameters are removed. ";
    }
    if (this.schema.hostsAllowed) {
      text += "Only the hosts matching " + this.schema.hostsAllowed + " are allowed. ";
    }
    if (this.schema.hostsDenied) {
      text += "But hosts matching " + this.schema.hostsAllowed + " are not allowed. ";
    }
    if (this.schema.allowProtocols) {
      text += "The protocol have to be: " + this.schema.allowProtocols + ". ";
    }
    if (!this.schema.allowRelative) {
      text += "Relative URLs are also allowed. ";
    }
    return cb(null, text);
  };

  exports.check = function(cb) {
    var i, j, len, len1, match, parts, ref, ref1, ref2, skip, success;
    skip = rules.optional.check.call(this);
    if (skip instanceof Error) {
      return cb(skip);
    }
    if (skip) {
      return cb();
    }
    if (typeof this.value !== 'string') {
      return this.sendError("The url has to be a string object but got " + (typeof this.value) + " instead", cb);
    }
    if (this.schema.toAbsoluteBase) {
      this.value = url.resolve(this.schema.toAbsoluteBase, this.value);
    }
    parts = url.parse(this.value);
    if (this.schema.removeQuery) {
      delete parts.search;
      delete parts.hash;
    }
    if (parts.host) {
      delete parts.host;
      if (this.schema.hostsAllowed) {
        success = true;
        if (!Array.isArray(this.schema.hostsAllowed)) {
          this.schema.hostsAllowed = [this.schema.hostsAllowed];
        }
        ref = this.schema.hostsAllowed;
        for (i = 0, len = ref.length; i < len; i++) {
          match = ref[i];
          if (match instanceof RegExp) {
            success = success && parts.hostname.match(match);
          } else {
            success = success && ~parts.hostname.indexOf(match);
          }
        }
        if (!success) {
          return this.sendError("The hostname '" + parts.hostname + "' should match against '" + this.schema.hostsAllowed + "'", cb);
        }
      }
      if (this.schema.hostsDenied) {
        success = true;
        if (!Array.isArray(this.schema.hostsDenied)) {
          this.schema.hostsDenied = [this.schema.hostsDenied];
        }
        ref1 = this.schema.hostsDenied;
        for (j = 0, len1 = ref1.length; j < len1; j++) {
          match = ref1[j];
          if (match instanceof RegExp) {
            success = success && !parts.hostname.match(match);
          } else {
            success = success && !~parts.hostname.indexOf(match);
          }
        }
        if (!success) {
          return this.sendError("The hostname '" + parts.hostname + "' should not match against '" + this.schema.hostsDenied + "'", cb);
        }
      }
    }
    if (this.schema.allowProtocols) {
      if (!parts.protocol) {
        return this.sendError("The protocol is missing", cb);
      }
      if (ref2 = parts.protocol.replace(/:/, ''), indexOf.call(this.schema.allowProtocols, ref2) < 0) {
        return this.sendError("The protocol " + parts.protocol + " is not allowed, only " + this.schema.allowProtocols, cb);
      }
    }
    if (!(this.schema.allowRelative || parts.hostname)) {
      return this.sendError("Relative URLs are not allowed", cb);
    }
    this.value = url.format(parts);
    return this.sendSuccess(cb);
  };

  exports.selfcheck = {
    title: "URL",
    description: "an url schema definition",
    type: 'object',
    allowedKeys: true,
    keys: util.extend({
      toAbsoluteBase: {
        title: "Absolute",
        description: "the base URL to make it absolute",
        type: 'url',
        optional: true
      },
      removeQuery: {
        title: "Remove Query",
        description: "a flag if set to `true` will remove the query string part",
        type: 'boolean',
        optional: true
      },
      hostsAllowed: {
        title: "Allowed Hosts",
        description: "the hosts allowed in the URL (white list)",
        type: 'or',
        optional: true,
        or: [
          {
            title: "Allowed Hosts List",
            description: "a list of all allowed hosts",
            type: 'array',
            toArray: true,
            minLength: 1,
            entries: {
              title: "Allowed Host",
              description: "an allowed host name",
              type: 'hostname'
            }
          }, {
            title: "Valid Hosts Match",
            description: "a regular expression for valid hosts",
            type: 'regexp'
          }
        ]
      },
      hostsDenied: {
        title: "Disallowed Hosts",
        description: "the hosts disallowed in the URL (black list)",
        type: 'or',
        optional: true,
        or: [
          {
            title: "Disallowed Hosts List",
            description: "a list of all disallowed hosts",
            type: 'array',
            toArray: true,
            minLength: 1,
            entries: {
              title: "Disallowed Host",
              description: "a disallowed host name",
              type: 'hostname'
            }
          }, {
            title: "Invalid Hosts Match",
            description: "a regular expression for invalid hosts",
            type: 'regexp'
          }
        ]
      },
      allowProtocols: {
        title: "Allowed Protocols",
        description: "a list of allowed protocols",
        type: 'array',
        toArray: true,
        minLength: 1,
        optional: true
      },
      allowRelative: {
        title: "Allow Relative URL",
        description: "a flag if set to `true` will also allow relative URLs",
        type: 'boolean',
        optional: true
      }
    }, rules.baseSchema, {
      "default": {
        title: "Default Value",
        description: "the default value to use if nothing given",
        type: 'url',
        allowRelative: true,
        optional: true
      }
    })
  };

}).call(this);

//# sourceMappingURL=url.map